import os.path
import sys
from gc import is_finalized

sys.path.insert(0, '../../../')
from gxl_ai_utils.utils import utils_file
#
# root_dir_lab = "/home/work_nfs15/asr_data/data/test_sets_format_3000"
# root_dir_huawei = "/mnt/sfs/asr/test_data/test_sets_format_3000"
# testsets = [
#     "public_test/roobo_100",
#     "caption",
#     "emotion", "public_test/MELD_test", "public_test/MER23_test",
#     "style",
#     "gender",  "public_test/aishell1_gender", "public_test/kaggle_gender",
#     "age",  "public_test/kaggle_age",
#     "chat","public_test/AirBench_speech",
# ]
# for test_set in testsets:
#     print("test_set: ", test_set)
#     input_path = f"{root_dir_lab}/{test_set}/data4huawei.list"
#     input_data_path = f"{root_dir_lab}/{test_set}/data.list"
#     dict_list = utils_file.load_dict_list_from_jsonl(input_data_path)
#     wav_list = []
#     new_dict_list = []
#     for item in dict_list:
#         wav_list.append(item["wav"])
#         item['wav'] = f"/mnt/sfs/asr/test_data/test_sets_format_3000/{test_set}/wav/{os.path.basename(item['wav'])}"
#         new_dict_list.append(item)
#     utils_file.write_dict_list_to_jsonl(new_dict_list, input_path)
#     output_data_path = f"{root_dir_huawei}/{test_set}/data.list"
#     input_text_path = f"{root_dir_lab}/{test_set}/text"
#     if True or test_set == "caption" or test_set == "emotion" or not utils_file.if_file_exist(input_text_path):
#         text_dict = {}
#         dict_list = utils_file.load_dict_list_from_jsonl(input_path)
#         for dict_i in dict_list:
#             text_dict[dict_i['key']] = dict_i['txt']
#         utils_file.write_dict_to_scp(text_dict, input_text_path)
#     output_text_path = f"{root_dir_huawei}/{test_set}/text"
#     utils_file.do_sync_copy_file_upload(input_path, output_data_path,
#                                         username="root",
#                                         password="Fy!mATB@QE",
#                                         remote_host="139.210.101.41",
#                                         )
#     utils_file.do_sync_copy_file_upload(input_text_path, output_text_path,
#                                         username="root",
#                                         password="Fy!mATB@QE",
#                                         remote_host="139.210.101.41",
#                                         )


# input_path = f"/home/xlgeng/.cache/tmp.list"
# input_text_path = f"/home/xlgeng/.cache/text.list"
# test_set = 'caption_aslp_record'
# input_data_path = f"/home/work_nfs9/yacao/nfs7_copy/yacao/1227_audio_tagging_wjtian/audio_caption_test_0113.list"
# dict_list = utils_file.load_dict_list_from_jsonl(input_data_path)
# wav_list = []
# new_dict_list = []
# for item in dict_list:
#     wav_list.append(item["wav"])
#     item['wav'] = f"/mnt/sfs/asr/test_data/test_sets_format_3000/{test_set}/wav/{os.path.basename(item['wav'])}"
#     new_dict_list.append(item)
# utils_file.write_dict_list_to_jsonl(new_dict_list, input_path)
#
# output_data_path = f"/mnt/sfs/asr/test_data/test_sets_format_3000/{test_set}/data.list"
# text_dict = {}
# dict_list = utils_file.load_dict_list_from_jsonl(input_path)
# for dict_i in dict_list:
#     text_dict[dict_i['key']] = dict_i['txt']
# utils_file.write_dict_to_scp(text_dict, input_text_path)
# output_text_path = f"/mnt/sfs/asr/test_data/test_sets_format_3000/{test_set}/text"
# utils_file.do_sync_copy_file_upload(input_path, output_data_path,
#                                     username="root",
#                                     password="Fy!mATB@QE",
#                                     remote_host="139.210.101.41",
#                                     )
# utils_file.do_sync_copy_file_upload(input_text_path, output_text_path,
#                                     username="root",
#                                     password="Fy!mATB@QE",
#                                     remote_host="139.210.101.41",
#                                     )
# time_now = utils_file.do_get_now_time()
# dict_list = utils_file.load_dict_list_from_jsonl(input_data_path)
# wav_path_list = []
# for dict_i in dict_list:
#     wav_path_list.append(dict_i['wav'])
# fake_path = utils_file.do_get_fake_file()
# utils_file.write_list_to_file(wav_path_list, fake_path)
#
# utils_file.do_sync_files_upload_data_multi_thread(
#     file_list_path=fake_path,
#     username="root",
#     password="Fy!mATB@QE",
#     remote_host="139.210.101.41",
#     remote_dir=f"/mnt/sfs/asr/test_data/test_sets_format_3000/{test_set}/wav",
#     num_thread=4
# )
# esl_time = utils_file.do_get_elapsed_time(time_now)
# utils_file.logging_info(f"Total time: {esl_time} s")



# time_now = utils_file.do_get_now_time()
# input_shards_path = "/home/work_nfs9/znlin/S2TCHAT/shards_list.txt"
# dataname = "asr_chat_znlin_2025-1-24"
# fake_file = utils_file.do_replace_str_for_file_and_return_new_file(input_shards_path,'/home/work_nfs9/znlin/S2TCHAT/shards',f"/mnt/sfs/asr/update_data/{dataname}")
# utils_file.do_sync_copy_file_upload(fake_file,f"/mnt/sfs/asr/update_data/{dataname}/shards_list.txt",
#                                     username="root",
#                                     password="Fy!mATB@QE",
#                                     remote_host="139.210.101.41",
#                                     )
# utils_file.do_sync_files_upload_data_multi_thread(
#     file_list_path=input_shards_path,
#     username="root",
#     password="Fy!mATB@QE",
#     remote_host="139.210.101.41",
#     remote_dir=f"/mnt/sfs/asr/update_data/{dataname}",
#     num_thread=10
# )
# esl_time = utils_file.do_get_elapsed_time(time_now)
# utils_file.logging_info(f"Total time: {esl_time} s")


# time_now = utils_file.do_get_now_time()
# input_shards_path = "/home/environment2/pkchen/cutwav/withid/shards_list.txt"
# dataname = "asr_chat_pkchen_2025-1-24"
# fake_file = utils_file.do_replace_str_for_file_and_return_new_file(input_shards_path,'/home/environment2/pkchen/cutwav/withid/shards',f"/mnt/sfs/asr/update_data/{dataname}")
# utils_file.do_sync_copy_file_upload(fake_file,f"/mnt/sfs/asr/update_data/{dataname}/shards_list.txt",
#                                     username="root",
#                                     password="Fy!mATB@QE",
#                                     remote_host="139.210.101.41",
#                                     )
# utils_file.do_sync_files_upload_data_multi_thread(
#     file_list_path=input_shards_path,
#     username="root",
#     password="Fy!mATB@QE",
#     remote_host="139.210.101.41",
#     remote_dir=f"/mnt/sfs/asr/update_data/{dataname}",
#     num_thread=10
# )
# esl_time = utils_file.do_get_elapsed_time(time_now)
# utils_file.logging_info(f"Total time: {esl_time} s")

# time_now = utils_file.do_get_now_time()
# input_shards_path = "/home/work_nfs15/asr_data/data/wenetspeech/wenetspeech_asr_chat_by_llm/shards/shards_list.txt"
# dataname = "asr_chat_wenetspeech_enhance_2025-1-24"
# fake_file = utils_file.do_replace_str_for_file_and_return_new_file(input_shards_path,'/home/work_nfs15/asr_data/data/wenetspeech/wenetspeech_asr_chat_by_llm/shards',f"/mnt/sfs/asr/update_data/{dataname}")
# utils_file.do_sync_copy_file_upload(fake_file,f"/mnt/sfs/asr/update_data/{dataname}/shards_list.txt",
#                                     username="root",
#                                     password="Fy!mATB@QE",
#                                     remote_host="139.210.101.41",
#                                     )
# utils_file.do_sync_files_upload_data_multi_thread(
#     file_list_path=input_shards_path,
#     username="root",
#     password="Fy!mATB@QE",
#     remote_host="139.210.101.41",
#     remote_dir=f"/mnt/sfs/asr/update_data/{dataname}",
#     num_thread=10
# )
# esl_time = utils_file.do_get_elapsed_time(time_now)
# utils_file.logging_info(f"Total time: {esl_time} s")


# time_now = utils_file.do_get_now_time()
# dataname = "TEXT2TOKEN_part_1"
# input_shards_path = f"/home/node48_tmpdata/hkxie/4O/speech_data_final/{dataname}/shards_list.txt"
# fake_file = utils_file.do_replace_str_for_file_and_return_new_file(input_shards_path,f'/home/node48_tmpdata/hkxie/4O/speech_data_final/{dataname}/shards',f"/mnt/sfs/asr/update_data/{dataname}")
# utils_file.do_sync_copy_file_upload(fake_file,f"/mnt/sfs/asr/update_data/{dataname}/shards_list.txt",
#                                     username="root",
#                                     password="Fy!mATB@QE",
#                                     remote_host="139.210.101.41",
#                                     )
# utils_file.do_sync_files_upload_data_multi_thread(
#     file_list_path=input_shards_path,
#     username="root",
#     password="Fy!mATB@QE",
#     remote_host="139.210.101.41",
#     remote_dir=f"/mnt/sfs/asr/update_data/{dataname}",
#     num_thread=8
# )
# esl_time = utils_file.do_get_elapsed_time(time_now)
# utils_file.logging_info(f"Total time: {esl_time} s")



# time_now = utils_file.do_get_now_time()
# dataname = "TEXT2TOKEN_part_2"
# input_shards_path = f"/home/node48_tmpdata/hkxie/4O/speech_data_final/{dataname}/shards_list.txt"
# fake_file = utils_file.do_replace_str_for_file_and_return_new_file(input_shards_path,f'/home/node48_tmpdata/hkxie/4O/speech_data_final/{dataname}/shards',f"/mnt/sfs/asr/update_data/{dataname}")
# utils_file.do_sync_copy_file_upload(fake_file,f"/mnt/sfs/asr/update_data/{dataname}/shards_list.txt",
#                                     username="root",
#                                     password="Fy!mATB@QE",
#                                     remote_host="139.210.101.41",
#                                     )
# utils_file.do_sync_files_upload_data_multi_thread(
#     file_list_path=input_shards_path,
#     username="root",
#     password="Fy!mATB@QE",
#     remote_host="139.210.101.41",
#     remote_dir=f"/mnt/sfs/asr/update_data/{dataname}",
#     num_thread=5
# )
# esl_time = utils_file.do_get_elapsed_time(time_now)
# utils_file.logging_info(f"Total time: {esl_time} s")


# time_now = utils_file.do_get_now_time()
# dataname = "TEXT2TOKEN_part_3"
# input_shards_path = f"/home/node48_tmpdata/hkxie/4O/speech_data_final/{dataname}/shards_list.txt"
# fake_file = utils_file.do_replace_str_for_file_and_return_new_file(input_shards_path,f'/home/node48_tmpdata/hkxie/4O/speech_data_final/{dataname}/shards',f"/mnt/sfs/asr/update_data/{dataname}")
# utils_file.do_sync_copy_file_upload(fake_file,f"/mnt/sfs/asr/update_data/{dataname}/shards_list.txt",
#                                     username="root",
#                                     password="Fy!mATB@QE",
#                                     remote_host="139.210.101.41",
#                                     )
# utils_file.do_sync_files_upload_data_multi_thread(
#     file_list_path=input_shards_path,
#     username="root",
#     password="Fy!mATB@QE",
#     remote_host="139.210.101.41",
#     remote_dir=f"/mnt/sfs/asr/update_data/{dataname}",
#     num_thread=5
# )
# esl_time = utils_file.do_get_elapsed_time(time_now)
# utils_file.logging_info(f"Total time: {esl_time} s")


#
# time_now = utils_file.do_get_now_time()
# input_shards_path = f"/home/node57_data2/tlzuo/dataset/DB-ASR-106_withid/speechtoken/shard.list"
# dataname = "speech2text_token_DB-ASR-106_add_by_2025-3_5"
# fake_file = utils_file.do_replace_str_for_file_and_return_new_file(input_shards_path,f'/home/node57_data2/tlzuo/dataset/DB-ASR-106_withid/speechtoken/shards',f"/mnt/sfs/asr/update_data/{dataname}")
# utils_file.do_sync_copy_file_upload(fake_file,f"/mnt/sfs/asr/update_data/{dataname}/shards_list.txt",
#                                     username="root",
#                                     password="Fy!mATB@QE",
#                                     remote_host="139.210.101.41",
#                                     )
# utils_file.do_sync_files_upload_data_multi_thread(
#     file_list_path=input_shards_path,
#     username="root",
#     password="Fy!mATB@QE",
#     remote_host="139.210.101.41",
#     remote_dir=f"/mnt/sfs/asr/update_data/{dataname}",
#     num_thread=10
# )
# esl_time = utils_file.do_get_elapsed_time(time_now)
# utils_file.logging_info(f"Total time: {esl_time} s")

time_now = utils_file.do_get_now_time()
root_input_dir = "/home/node48_tmpdata/hkxie/4O/speech_data_final/TEXT2TOKEN_part_5"
input_shards_path = f"{root_input_dir}/shards_list.txt"
dataname = "TEXT2TOKEN_part_5"
remote_root_dir="/mnt/sfs/asr/update_data"
fake_file = utils_file.do_replace_str_for_file_and_return_new_file(input_shards_path,f'{root_input_dir}/shards',f"{remote_root_dir}/{dataname}")
utils_file.do_sync_copy_file_upload(fake_file,f"{remote_root_dir}/{dataname}/shards_list.txt",
                                    username="root",
                                    password="Fy!mATB@QE",
                                    remote_host="139.210.101.41",
                                    )
utils_file.do_sync_files_upload_data_multi_thread(
    file_list_path=input_shards_path,
    username="root",
    password="Fy!mATB@QE",
    remote_host="139.210.101.41",
    remote_dir=f"{remote_root_dir}/{dataname}",
    num_thread=5 # 30h
)
esl_time = utils_file.do_get_elapsed_time(time_now)
utils_file.logging_info(f"Total time: {esl_time} s")

time_now = utils_file.do_get_now_time()
root_input_dir = "/home/node48_tmpdata/hkxie/4O/speech_data_final/TEXT2TOKEN_part_6"
input_shards_path = f"{root_input_dir}/shards_list.txt"
dataname = "TEXT2TOKEN_part_6"
remote_root_dir="/mnt/sfs/asr/update_data"
fake_file = utils_file.do_replace_str_for_file_and_return_new_file(input_shards_path,f'{root_input_dir}/shards',f"{remote_root_dir}/{dataname}")
utils_file.do_sync_copy_file_upload(fake_file,f"{remote_root_dir}/{dataname}/shards_list.txt",
                                    username="root",
                                    password="Fy!mATB@QE",
                                    remote_host="139.210.101.41",
                                    )
utils_file.do_sync_files_upload_data_multi_thread(
    file_list_path=input_shards_path,
    username="root",
    password="Fy!mATB@QE",
    remote_host="139.210.101.41",
    remote_dir=f"{remote_root_dir}/{dataname}",
    num_thread=5 # 30h
)
esl_time = utils_file.do_get_elapsed_time(time_now)
utils_file.logging_info(f"Total time: {esl_time} s")

time_now = utils_file.do_get_now_time()
root_input_dir = "/home/node48_tmpdata/hkxie/4O/speech_data_final/TEXT2TOKEN_part_7"
input_shards_path = f"{root_input_dir}/shards_list.txt"
dataname = "TEXT2TOKEN_part_7"
remote_root_dir="/mnt/sfs/asr/update_data"
fake_file = utils_file.do_replace_str_for_file_and_return_new_file(input_shards_path,f'{root_input_dir}/shards',f"{remote_root_dir}/{dataname}")
utils_file.do_sync_copy_file_upload(fake_file,f"{remote_root_dir}/{dataname}/shards_list.txt",
                                    username="root",
                                    password="Fy!mATB@QE",
                                    remote_host="139.210.101.41",
                                    )
utils_file.do_sync_files_upload_data_multi_thread(
    file_list_path=input_shards_path,
    username="root",
    password="Fy!mATB@QE",
    remote_host="139.210.101.41",
    remote_dir=f"{remote_root_dir}/{dataname}",
    num_thread=5 # 30h
)
esl_time = utils_file.do_get_elapsed_time(time_now)
utils_file.logging_info(f"Total time: {esl_time} s")

time_now = utils_file.do_get_now_time()
root_input_dir = "/home/node48_tmpdata/hkxie/4O/speech_data_final/TEXT2TOKEN_part_8"
input_shards_path = f"{root_input_dir}/shards_list.txt"
dataname = "TEXT2TOKEN_part_8"
remote_root_dir="/mnt/sfs/asr/update_data"
fake_file = utils_file.do_replace_str_for_file_and_return_new_file(input_shards_path,f'{root_input_dir}/shards',f"{remote_root_dir}/{dataname}")
utils_file.do_sync_copy_file_upload(fake_file,f"{remote_root_dir}/{dataname}/shards_list.txt",
                                    username="root",
                                    password="Fy!mATB@QE",
                                    remote_host="139.210.101.41",
                                    )
utils_file.do_sync_files_upload_data_multi_thread(
    file_list_path=input_shards_path,
    username="root",
    password="Fy!mATB@QE",
    remote_host="139.210.101.41",
    remote_dir=f"{remote_root_dir}/{dataname}",
    num_thread=5 # 30h
)
esl_time = utils_file.do_get_elapsed_time(time_now)
utils_file.logging_info(f"Total time: {esl_time} s")


import jieba
def get_word_indices(tokens, token_list):
    word_indices = []
    index = 0
    for token in tokens:
        word_indices.append(index)
        index += len(token)
        while index < len(token_list) and token_list[index] == '[]':
            index += 1
    return word_indices

sentence = "我来自中国"
token_list = ['我','[]','[]','来','自','[]','中','国','[]']

# 使用jieba进行分词
tokens = list(jieba.cut(sentence))

# 获取每个词占有的列表中字符的索引
word_indices = get_word_indices(tokens, token_list)

print("分词结果：", tokens)
print("每个词占有的列表中字符的索引：", word_indices)


import tqdm

from gxl_ai_utils.utils import utils_file

from modelscope.pipelines import pipeline
from modelscope.utils.constant import Tasks
import os
import sys

arg_num = len(sys.argv)

input_wav_scp_path = sys.argv[1]
output_dir = sys.argv[2]
if len(sys.argv) > 3:
    lab_text_path = sys.argv[3]
else:
    lab_text_path = None

utils_file.do_inference_paraformer(input_wav_scp_path, output_dir, lab_text_path)



import re

import jieba.posseg as pseg
import thulac


def extract_nouns(string_list):
    nouns = set()
    nouns_dict = {}
    for string in string_list:
        words = pseg.cut(string)
        for word, flag in words:
            if flag.startswith('n'):  # 以'n'开头的词性标记表示名词
                nouns.add(word)
                if word in nouns_dict:
                    nouns_dict[word] += 1
                else:
                    nouns_dict[word] = 1
    return nouns, nouns_dict


def convert_to_indices(s):
    result = []
    index = 0
    for word in s:
        if re.match("^[A-Za-z]*$", word):
            result.append((index,))
            index += 1
        else:
            result.append(tuple(range(index, index + len(word))))
            index += len(word)
    return result
def do_fenci(string):
        words =  pseg.cut(string)
        res = []
        for word, flag in words:
            res.append(word)
        return res, convert_to_indices(res)


thu1 = thulac.thulac(seg_only=True)  # 设置只进行分词，不进行词性标注

def do_fenci(string):
    string = re.sub(r'([\u4e00-\u9fa5])([a-zA-Z])', r'\1 \2', string)
    string = re.sub(r'([a-zA-Z])([\u4e00-\u9fa5])', r'\1 \2', string)
    words = thu1.cut(string, text=True)  # 进行分词
    res = words.split(' ')  # 分词结果是一个字符串，每个词之间用空格分隔，所以我们需要用split函数来将其转换为列表
    return res, convert_to_indices(res)

if __name__ == '__main__':
    test_str_list = [
        "我爱吃苹果hello嘻嘻并且喜欢和妲己一起玩耍",
        "我们一起举办了一场盛大的庆祝活动大家欢笑着共度美好时光",
        "他是一个有着丰富经验和深厚知识的专家在行业内享有很高的声誉",
    ]

    for test_str in test_str_list:
        print(test_str)
        print(do_fenci(test_str))


import argparse
import glob
import os
import threading
import tqdm
import sys

sys.path.insert(0, '../../../')

from gxl_ai_utils.utils import utils_file

NEW2OLD_TABLE = {}
VAD_INFO_TABLE = {}
THREAD_NUM = 40


def get_vad_info_list(vad_info_txt_path):
    """
    从vad_info.txt中读取时间戳， 得到排过序的一个双层列表 [[s,e],[s,e],[s,e]...]
    :param vad_info_txt_path:
    :return:
    """

    vad_info_str_list = utils_file.load_list_file_clean(vad_info_txt_path)
    vad_info_list = []
    for vad_info_str in vad_info_str_list:
        vad_i = vad_info_str.strip().split(",")
        vad_info_list.append(vad_i)
    sorted_list = sorted(vad_info_list, key=lambda x: int(x[0]))
    return sorted_list


def handle_path(input_new_tts_final_path, only_name=False):
    """"""
    # 郭钊版：/root/path/282492425329_CPKwq_86_6299.wav
    if not only_name:
        file_name = utils_file.get_file_pure_name_from_path(input_new_tts_final_path)
    else:
        file_name = input_new_tts_final_path
    file_name_item_list = file_name.strip().split("_")
    parent_new_name = file_name_item_list[0] + "_" + file_name_item_list[1]
    index = int(file_name_item_list[2])
    millisecond_num = int(file_name_item_list[3])
    return parent_new_name, index, millisecond_num


def get_true_wav_path_by_index(output_dir, parent_new_name, index, millisecond_num):
    """"""
    # 郭钊版:/root/path/282492425329_CPKwq_86_6299.wav
    file_name = parent_new_name + "_" + str(index) + "_" + str(millisecond_num) + ".wav"
    file_name_rough = parent_new_name + "_" + str(index) + "_" + "*" + ".wav"
    true_wav_path_list = glob.glob(os.path.join(output_dir, file_name_rough))
    true_wav_path = true_wav_path_list[0] if len(true_wav_path_list) > 0 else None
    if true_wav_path is None:
        utils_file.logging_print(
            f"fix_path(): true_wav_path is None, parent_new_name={parent_new_name}, index={index}, millisecond_num={millisecond_num}"
        )
        return None
    _, _, duration = handle_path(true_wav_path)
    if abs(duration - millisecond_num) < 10:
        return true_wav_path
    else:
        utils_file.logging_print(
            f"fix_path(): abs(duration - millisecond_num) > 10={abs(duration - millisecond_num)}, 采用真实的音频地址")
        return true_wav_path


def do_handle_wav(wav_new_path, output_dir="./"):
    """"""
    # 首先得到小段音频对应长短音频的名字。282492425329_CPKwq_86_6299.txt
    little_new_name = utils_file.get_file_pure_name_from_path(wav_new_path)
    temp_name_apart = little_new_name.strip().split("_")
    little_new_name = temp_name_apart[0] + "_" + temp_name_apart[1]
    the_old_wav_path = NEW2OLD_TABLE.get(little_new_name, None)
    if the_old_wav_path is None:
        utils_file.logging_print(
            f"do_handle_wav(): the_old_wav_path is None, little_new_name={little_new_name}"
        )
        return False
    if not os.path.exists(the_old_wav_path):
        utils_file.logging_print(
            f"do_handle_wav(): the_old_wav_path is not exist, little_new_name={little_new_name}"
        )
        return False
    long_old_clean_wav_dir = os.path.join(output_dir, "temp_long_old_clean_wav")
    utils_file.makedir_sil(long_old_clean_wav_dir)
    long_old_clean_path = os.path.join(long_old_clean_wav_dir, little_new_name + ".wav")
    if not os.path.exists(long_old_clean_path):
        # 先规范化音频
        clean_wav(the_old_wav_path, long_old_clean_path)
        # 开始切割音频
        vad_info_txt_path = VAD_INFO_TABLE.get(little_new_name, None)
        if vad_info_txt_path is None:
            utils_file.logging_print(
                f"do_handle_wav(): vad_info_txt_path is None, little_new_name={little_new_name}"
            )
            return False
        final_wav_dir = os.path.join(output_dir, "final_wav")
        utils_file.makedir_sil(final_wav_dir)
        slicing_wav(long_old_clean_path, vad_info_txt_path, final_wav_dir, little_new_name)
    else:
        utils_file.logging_print(
            f"do_handle_wav(): long_old_clean_path is exist, little_new_name={little_new_name}, 跳过该长音频的处理"
        )
    return True


def clean_wav(input_file_path, output_file_path):
    """
    将音频整理成标准格式， 16K采样， 单通道，补齐音频头
    :param input_file_path:
    :param output_file_path:
    :return:
    """
    os.system(f"ffmpeg -i '{input_file_path}' -ac 1 -ar 16000 -vn {output_file_path}")


def little_func4slicing_wav(i, vad_info, input_file_path, output_dir, wav_new_name):
    # print(vad_info)
    start_time = vad_info[0]
    end_time = vad_info[1]
    duration = int(end_time) - int(start_time)
    start_sample = int(start_time) * 16
    end_sample = int(end_time) * 16
    output_path = os.path.join(output_dir, f"{wav_new_name}_{i}_{duration}.wav")
    utils_file.do_extract_audio_segment(input_file_path, output_path, start_sample, end_sample)


def slicing_wav(input_file_path, vad_info_txt_path, output_dir, wav_new_name):
    """
    将一个音频,依据时间戳, 切分成若干小音频. 此处可以多线程并行处理
    """
    vad_info_str_list = utils_file.load_list_file_clean(vad_info_txt_path)
    vad_info_list = []
    sorted_list = get_vad_info_list(vad_info_txt_path)
    thread_num = THREAD_NUM if len(sorted_list) > THREAD_NUM else len(sorted_list)
    runner = utils_file.GxlFixedThreadPool(thread_num)
    for i, vad_info in enumerate(sorted_list):
        i = i + 1
        runner.add_thread(little_func4slicing_wav, [i, vad_info, input_file_path, output_dir, wav_new_name])
    runner.start()


def do_get_vad_scp_file(input_dir, output_dir='./'):
    """
    tts处理流程中会对每一个音频生成一个vad_info的txt文件, 我们得到一个key vad_info.txt的字典
    :param input_dir:
    :return:
    """
    vad_res_scp = utils_file.get_scp_for_wav_dir(input_dir, suffix="txt")
    utils_file.write_dict_to_scp(vad_res_scp, os.path.join(output_dir, "vad_res.scp"))


def do_get_old2new_scp_file(input_dir, output_dir='./'):
    """
    这个input_dir中包含大量的old2new_*.txt文件, 我们得到一个key old2new.txt的字典
    :param input_dir:
    :return:
    """
    if os.path.isfile(input_dir):
        utils_file.logging_print("do_get_old2new_scp_file(): input_dir是文件")
        utils_file.copy_file(input_dir, os.path.join(output_dir, "old2new.scp"))
        return
    old2new_path_list = glob.glob(f"{input_dir}/old2new*.scp")
    res_dict = {}
    for old2new_path in old2new_path_list:
        old2new_dict = utils_file.load_dict_from_scp(old2new_path)
        res_dict.update(old2new_dict)
    little_res_dict = utils_file.get_subdict(res_dict, 0, 10)
    utils_file.logging_print("do_get_old2new_scp_file(): 小字典示例:")
    utils_file.print_dict(little_res_dict)
    utils_file.write_dict_to_scp(res_dict, os.path.join(output_dir, "old2new.scp"))


def do_get_final_jsonl(input_scp_path, output_dir='./'):
    """
    多线程实现
    :param input_scp_path:
    :param output_dir:
    :return:
    """
    final_dict = utils_file.load_dict_from_scp(input_scp_path)
    thread_num = 6  # 暂时不支持多线程， 多线程没法保证不同时切割同一条长音频。
    runner = utils_file.GxlDynamicThreadPool()
    little_dict_list = utils_file.do_split_dict(final_dict, thread_num)
    # lock4write = threading.Lock()
    asr_final_list_path = os.path.join(output_dir, "asr_final_list.txt")
    if os.path.exists(asr_final_list_path):
        # 此处必须， 后面逻辑是在文本末尾累加写入，一旦程序中断重新运行，如果不删原来的，
        # 就会出问题， 文件写入内容本身不占用太多时间，时间主要是在于切分以您，此处已做了
        # 防中断处理。可放心随意重新运行程序， 而不会重复工作。
        os.remove(asr_final_list_path)
    for i, little_final_dict in enumerate(little_dict_list):
        # 此处的多线程不太长成熟， 一旦多个线程同时处理同一个长音频的小音频，就会同时对该长音频的原始音频做再切割，
        # 而我们是只需要切一次， 不是多次。在切音频时仍然可以多线程， 此处无多线程也无所谓。
        runner.add_task(little_func4get_final_jsonl,
                        [i, little_final_dict, asr_final_list_path, "None", output_dir])
    runner.start()
    utils_file.logging_print("do_get_final_jsonl(): 结束")
    utils_file.logging_print('开始整合')
    asr_final_list_path_format = asr_final_list_path.replace(".txt", "")
    asr_final_path_list = glob.glob(f"{asr_final_list_path_format}*")
    res_list = []
    utils_file.print_list(asr_final_path_list)
    for asr_final_path in asr_final_path_list:
        little_dict_list = utils_file.load_dict_list_from_jsonl(asr_final_path)
        res_list.extend(little_dict_list)
    utils_file.write_dict_list_to_jsonl(res_list, asr_final_list_path)
    utils_file.logging_print('整合结束')


def little_func4get_final_jsonl(i, final_dict, asr_final_jsonl_path, lock4write, output_dir):
    asr_final_jsonl_path = asr_final_jsonl_path.replace(".txt", f"_{i}.txt")
    if os.path.exists(asr_final_jsonl_path):
        # 此处必须， 后面逻辑是在文本末尾累加写入，一旦程序中断重新运行，如果不删原来的，
        # 就会出问题， 文件写入内容本身不占用太多时间，时间主要是在于切分以您，此处已做了
        # 防中断处理。可放心随意重新运行程序， 而不会重复工作。
        os.remove(asr_final_jsonl_path)
    for key, value in tqdm.tqdm(final_dict.items(), total=len(final_dict)):
        value_list = value.strip().split(r' ')
        if len(value_list) != 2:
            utils_file.logging_print(f"key:{key} value:{value}, len(value_list) != 2")
        wav_path = value_list[0]
        txt_path = value_list[1]
        txt = utils_file.load_first_row_clean(txt_path)
        if len(txt) == 0:
            utils_file.logging_print(f"txt_path文件内部无内容， key:{key} wav_path:{wav_path} txt_path:{txt_path}")
            continue
        txt = txt.strip().split("\t")[1]
        res_handle_wav = do_handle_wav(wav_path, output_dir)
        if not res_handle_wav:
            continue
        parent_new_filename, index, duration = handle_path(wav_path)
        asr_final_wav_dir = os.path.join(output_dir, "final_wav")
        wav_path = get_true_wav_path_by_index(asr_final_wav_dir, parent_new_filename, index, duration)
        if wav_path is None or not os.path.exists(wav_path):
            continue
        dict_i = dict(key=key, wav=wav_path, txt=txt)
        # with lock4write:
        utils_file.logging_print(f'开始写入文件:{asr_final_jsonl_path}')
        utils_file.write_single_dict_to_jsonl(dict_i, asr_final_jsonl_path)
        utils_file.logging_print(f'结束写入文件:{asr_final_jsonl_path}')
    utils_file.logging_print("little_func4get_final_jsonl(): 结束")


def main(old2new_dir, input_vad_info_dir, final_scp_path, output_dir):
    """"""
    utils_file.makedir_sil(output_dir)
    do_get_vad_scp_file(input_vad_info_dir, output_dir)
    do_get_old2new_scp_file(old2new_dir, output_dir)
    NEW2OLD_TABLE.update(utils_file.load_dict_from_scp(os.path.join(output_dir, "old2new.scp")))
    VAD_INFO_TABLE.update(utils_file.load_dict_from_scp(os.path.join(output_dir, "./vad_res.scp")))
    do_get_final_jsonl(final_scp_path,
                       output_dir=output_dir)


def split_source(input_tts_scp_path, output_dir, split_ratio=None):
    """将一个scp文件分成若干个小文件"""
    if split_ratio is None:
        split_ratio = [0.45, 0.55]
    all_dict = utils_file.load_dict_from_scp(input_tts_scp_path)
    total_ratio = sum(split_ratio)
    if total_ratio != 1.0:
        raise ValueError("Ratios should add up to 1.0")
    start = 0
    for i, ratio in enumerate(split_ratio):
        end = start + int(len(all_dict) * ratio)
        dict_i = utils_file.get_subdict(all_dict, start, end)
        start = end
        utils_file.write_dict_to_scp(dict_i, os.path.join(output_dir, f"final_scp_part_{i}.scp"))


def get_little_final(input_final_scp_path, output_dir):
    """从非常大的tts_final_scp中取出一小点，组成一个测试用的小的tts_final_scp文件"""
    all_dict = utils_file.load_dict_from_scp(input_final_scp_path)
    little_dict = utils_file.get_random_subdict(all_dict, 25)
    utils_file.write_dict_to_scp(little_dict, os.path.join(output_dir, f"final_scp_little.scp"))


def fix_old2new_file():
    """
    将2T的old2new.scp进行修复
    :return:
    """
    old_scp_path = "/home/node39_data2/zhguo/data_hot_topic/list/init_lists/old2new.scp"
    old_dict = utils_file.load_dict_from_scp(old_scp_path)
    new_dict = {}
    for key, value in tqdm.tqdm(old_dict.items(), total=len(old_dict)):
        value = value.replace("/home/work_nfs6/xlgeng/data/pachong_ximalaya_2/wav_files/",
                              "/home/node36_data/ximalaya_2T/")
        new_dict[key] = value
    new_scp_path = "/home/node36_data/xlgeng/asr_data_from_pachong/xmly2T_hot_point_old2new.scp"
    utils_file.write_dict_to_scp(new_dict, new_scp_path)

    old_scp_path = "/home/backup_nfs5/zhguo/audio_drama/list/init_lists/old2new.scp"
    old_dict = utils_file.load_dict_from_scp(old_scp_path)
    new_dict = {}
    for key, value in tqdm.tqdm(old_dict.items(), total=len(old_dict)):
        value = value.replace("/home/work_nfs6/xlgeng/data/pachong_ximalaya_2/wav_files/",
                              "/home/node36_data/ximalaya_2T/")
        new_dict[key] = value
    new_scp_path = "/home/node36_data/xlgeng/asr_data_from_pachong/xmly2T_radio_drama_old2new.scp"
    utils_file.write_dict_to_scp(new_dict, new_scp_path)


print('---------------------------------------------------')


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--tts_final_scp_path", type=str)
    parser.add_argument("--asr_final_scp_path", type=str)
    parser.add_argument("--do_split", type=bool, default=False)
    args = parser.parse_args()
    return args


def pre_do(final_tts_scp_path: str, output_dir: str):
    utils_file.logging_print('首先切分tts_final.scp,切分为6个')
    # output_dir = "/home/node36_data/xlgeng/asr_data_from_pachong/gxl_output/ximalaya_lishi_10T"
    # final_tts_scp_path = "/home/work_nfs8/lhma/double_check_lists/zhguo_lishi_part0_3700h.scp"
    final_tts_dict = utils_file.load_dict_from_scp(final_tts_scp_path)
    tts_dict_list = utils_file.do_split_dict(final_tts_dict, 6)
    for i, tts_dict in enumerate(tts_dict_list):
        i = i + 1
        utils_file.write_dict_to_scp(tts_dict, os.path.join(output_dir, f"tts_final_{i}.scp"))


def do_handle_wav_2(long_wav_new_name, output_dir="./"):
    """"""
    global NEW2OLD_TABLE, VAD_INFO_TABLE
    the_old_wav_path = NEW2OLD_TABLE.get(long_wav_new_name, None)
    if the_old_wav_path is None:
        utils_file.logging_print(
            f"do_handle_wav(): the_old_wav_path is None, long_wav_new_name={long_wav_new_name}"
        )
        return False
    if not os.path.exists(the_old_wav_path):
        utils_file.logging_print(
            f"do_handle_wav(): the_old_wav_path is not exist, long_wav_new_name={long_wav_new_name}"
        )
        return False
    long_old_clean_wav_dir = os.path.join(output_dir, "temp_long_old_clean_wav")
    utils_file.makedir_sil(long_old_clean_wav_dir)
    long_clean_path = os.path.join(long_old_clean_wav_dir, long_wav_new_name + ".wav")
    if not os.path.exists(long_clean_path):
        # 先规范化音频
        clean_wav(the_old_wav_path, long_clean_path)
        # 开始切割音频
        vad_info_txt_path = VAD_INFO_TABLE.get(long_wav_new_name, None)
        if vad_info_txt_path is None:
            utils_file.logging_print(
                f"do_handle_wav(): vad_info_txt_path is None, long_wav_new_name={long_wav_new_name}"
            )
            return False
        final_wav_dir = os.path.join(output_dir, "final_wav")
        utils_file.makedir_sil(final_wav_dir)
        slicing_wav(long_clean_path, vad_info_txt_path, final_wav_dir, long_wav_new_name)
    else:
        utils_file.logging_print(
            f"do_handle_wav(): long_old_clean_path is exist, long_wav_new_name={long_wav_new_name}, 跳过该长音频的处理"
        )
    return True


def solution_2(output_dir, old2new_dir, input_vad_info_dir):
    utils_file.logging_print('首先从命令行中拿到要处理的tts_final.scp的子集')
    args = get_args()
    if args.do_split:
        utils_file.logging_print('开始do_split')
        pre_do(final_tts_scp_path, output_dir)
        utils_file.logging_print('do_split完成, 直接return')
        return
    tts_scp_path = os.path.join(output_dir, args.tts_final_scp_path)
    asr_final_path = os.path.join(output_dir, args.asr_final_scp_path)
    if os.path.exists(asr_final_path):
        utils_file.logging_print('asr_scp_path已经存在，直接删除')
        os.remove(asr_final_path)
    utils_file.logging_print('加载vad和old2new的信息')
    utils_file.makedir_sil(output_dir)
    do_get_vad_scp_file(input_vad_info_dir, output_dir)
    do_get_old2new_scp_file(old2new_dir, output_dir)
    NEW2OLD_TABLE.update(utils_file.load_dict_from_scp(os.path.join(output_dir, "old2new.scp")))
    VAD_INFO_TABLE.update(utils_file.load_dict_from_scp(os.path.join(output_dir, "./vad_res.scp")))
    utils_file.logging_print('获取tts_scp对应的长音频的新名字列表')
    tts_dict = utils_file.load_dict_from_scp(tts_scp_path)
    long_wav_new_names_list = []
    for key in tqdm.tqdm(tts_dict.keys(), total=len(tts_dict)):
        long_wav_new_name, _, _ = handle_path(key, only_name=True)
        long_wav_new_names_list.append(long_wav_new_name)
    long_wav_new_names_list = list(set(long_wav_new_names_list))
    utils_file.logging_print("开始逐条长音频得处理")
    for long_wav_new_name in tqdm.tqdm(long_wav_new_names_list, total=len(long_wav_new_names_list)):
        do_handle_wav(long_wav_new_name, output_dir)
    utils_file.logging_print("开始逐tts_final条映射到asr_final, 需要进行改进")

    utils_file.logging_print('开始得到文本字典')
    asr_text_path = asr_final_path.replace('.list', '.text')
    text_dict = {}
    # if os.path.exists(asr_text_path):
    #     utils_file.logging_print('asr_text_path已经存在，无需再处理')
    # else:
    # utils_file.logging_print('asr_text_path不存在')
    utils_file.makedir_sil(os.path.dirname(asr_text_path))
    for key, value in tqdm.tqdm(tts_dict.items(), total=len(tts_dict)):
        value_list = value.strip().split(r' ')
        if len(value_list) != 2:
            utils_file.logging_print(f"key:{key} value:{value},len(value_list) != 2")
        wav_path = value_list[0]
        txt_path = value_list[1]
        if not os.path.exists(txt_path):
            continue
        txt = utils_file.load_first_row_clean(txt_path)
        if len(txt) == 0:
            utils_file.logging_print(f"txt_path文件内部无内容， key:{key} wav_path:{wav_path} txt_path:{txt_path}")
            continue
        txt = txt.strip().split("\t")[1]
        text_dict[key] = txt
    utils_file.write_dict_to_scp(text_dict, asr_text_path)

    utils_file.logging_print('开始得到wav字典')
    wav_dict = utils_file.get_scp_for_wav_dir(os.path.join(output_dir, "final_wav"))
    wav_dict_keys = list(wav_dict.keys())
    my_wav_dict = {}
    asr_wavscp_path = asr_final_path.replace('.list', '_wav.scp')
    # if os.path.exists(asr_wavscp_path):
    #     utils_file.logging_print('asr_wavscp_path已经存在，无需再处理')
    # else:
    # utils_file.logging_print('asr_wavscp_path不存在')
    for key, value in tqdm.tqdm(tts_dict.items(), total=len(tts_dict)):
        value_list = value.strip().split(r' ')
        if len(value_list) != 2:
            utils_file.logging_print(f"key:{key} value:{value},len(value_list) != 2")
        wav_path = value_list[0]
        txt_path = value_list[1]
        parent_new_filename, index, duration = handle_path(wav_path)
        key4wav_dict = next((s for s in wav_dict_keys if s.startswith(f"{parent_new_filename}_{index}")), None)
        if key4wav_dict is None:
            continue
        wav_path_true = wav_dict[key4wav_dict]
        if wav_path_true is None or not os.path.exists(wav_path_true):
            continue
        my_wav_dict[key] = wav_path_true
    utils_file.write_dict_to_scp(my_wav_dict, asr_wavscp_path)
    utils_file.do_convert_wav_text_scp_to_jsonl(asr_wavscp_path, asr_text_path, asr_final_path)

    utils_file.logging_print("处理完毕")


def final_handle():
    """"""
    output_dir = "/home/node36_data/xlgeng/asr_data_from_pachong/gxl_output/ximalaya_lishi_10T"
    text_file_list = glob.glob(os.path.join(output_dir, "*.text"))
    text_dict_all = {}
    for text_file in text_file_list:
        text_dict = utils_file.load_dict_from_scp(text_file)
        text_dict_all.update(text_dict)
    utils_file.write_dict_to_scp(text_dict_all, os.path.join(output_dir, "all.text"))
    wav_dict_all = utils_file.get_scp_for_wav_dir(os.path.join(output_dir, "final_wav"))
    utils_file.write_dict_to_scp(wav_dict_all, os.path.join(output_dir, "all_wav.scp"))
    utils_file.do_convert_wav_text_scp_to_jsonl(os.path.join(output_dir, "all_wav.scp"),
                                                os.path.join(output_dir, "all.text"),
                                                os.path.join(output_dir, "all.list"))

def copy_file_to_nfs():
    """"""
    source_list_path = "/home/node36_data/xlgeng/asr_data_from_pachong/gxl_output/ximalaya_lishi_10T/all.list"
    dict_list = utils_file.load_dict_list_from_jsonl(source_list_path)
    target_dir = "/home/work_nfs14/xlgeng/asr_data_from_pachong/ximalaya_lishi_10T"
    utils_file.makedir_sil(target_dir)
    runner = utils_file.GxlFixedThreadPool(50)
    utils_file.logging_print('得到wav path list')
    wav_path_list = []
    for item in tqdm.tqdm(dict_list):
        wav_path = item['wav']
        wav_path_list.append(wav_path)
    runner.map(utils_file.copy_file2, wav_path_list, {"target_dir": target_dir})
    runner.start()

if __name__ == '__main__':
    # output_dir = "/home/node36_data/xlgeng/asr_data_from_pachong/xmly10T-history-0/mcshao"
    # # output_dir ='./'
    # input_vad_info_dir = "/home/node36_data/zhguo/history_10T/part_0/vad/txts"
    # # 表格里写的如果不是dir,而是成品scp文件， 就把scp文件的路径传入即可
    # old2new_dir = "/home/node36_data/zhguo/history_10T/part_0/list/init_lists/"
    # final_tts_scp_path = "/home/work_nfs10/mcshao/TTS-ASR-PROCESS/Splited_source/aa/final_scp_part_0.scp"
    # main(old2new_dir, input_vad_info_dir, final_tts_scp_path, output_dir)
    # # split_source(final_tts_scp_path, './')
    # # get_little_final(final_tts_scp_path, './')

    # output_dir = './data_output/lishi-1'
    output_dir = "/home/node36_data/xlgeng/asr_data_from_pachong/gxl_output/ximalaya_lishi_10T"
    utils_file.makedir_for_file_or_dir(output_dir)

    input_vad_info_dir = "/home/node36_data/zhguo/history_10T/part_0/vad/txts"
    # input_vad_info_dir = "/home/work_nfs9/zhguo/data/history_10T/part_1/vad/txts"
    # input_vad_info_dir = "/home/node39_data2/zhguo/data_hot_topic/lists/vad_result"
    # input_vad_info_dir = "/home/node36_data/zhguo/audio_drama/vad/txts"

    # 表格里写的如果不是dir,而是成品scp文件， 就把scp文件的路径传入即可
    old2new_dir = "/home/node36_data/zhguo/history_10T/part_0/list/init_lists"
    # old2new_dir = "/home/node36_data/zhguo/history_10T/part_1/list/init_lists"
    # old2new_dir = "/home/node36_data/xlgeng/asr_data_from_pachong/xmly2T_hot_point_old2new.scp"
    # old2new_dir = "/home/node36_data/xlgeng/asr_data_from_pachong/xmly2T_radio_drama_old2new.scp"

    final_tts_scp_path = "/home/work_nfs8/lhma/double_check_lists/zhguo_lishi_part0_3700h.scp"
    # final_tts_scp_path = "/home/work_nfs14/zhguo/data/history_10T/part_1/final2.scp"
    # final_tts_scp_path = "/home/work_nfs8/lhma/double_check_lists/zhguo_xmly2T_redian_950h.scp"
    # final_tts_scp_path = "/home/work_nfs8/lhma/double_check_lists/zhguo_xmly2T_guangbojv_1550h.scp"

    # get_little_final(final_tts_scp_path, './')
    # final_tts_scp_path = "/home/work_nfs8/xlgeng/new_workspace/gxl_ai_utils/eggs/cats_and_dogs/handle_tts_data_for_asr/final_scp_little.scp"
    # main(old2new_dir, input_vad_info_dir, final_tts_scp_path, output_dir)
    # solution_2(output_dir, old2new_dir, input_vad_info_dir)
    copy_file_to_nfs()



# -*- coding: utf-8 -*-
"""
Created on Fri Nov  1 10:28:41 2019

@author: rocheng
"""
import os
import csv
from shutil import copyfile
import glob


def get_dir(cfg, param_name, new_dir_name):
    '''Helper function to retrieve directory name if it exists,
       create it if it doesn't exist'''

    if param_name in cfg:
        dir_name = cfg[param_name]
    else:
        dir_name = os.path.join(os.path.dirname(__file__), new_dir_name)
    if not os.path.exists(dir_name):
        os.makedirs(dir_name)
    return dir_name


def write_log_file(log_dir, log_filename, data):
    '''Helper function to write log file'''
    data = zip(*data)
    with open(os.path.join(log_dir, log_filename), mode='w', newline='') as csvfile:
        csvwriter = csv.writer(csvfile, delimiter=' ',
                               quotechar='|', quoting=csv.QUOTE_MINIMAL)
        for row in data:
            csvwriter.writerow([row])


def str2bool(string):
    return string.lower() in ("yes", "true", "t", "1")


def rename_copyfile(src_path, dest_dir, prefix='', ext='*.wav'):
    srcfiles = glob.glob(f"{src_path}/" + ext)
    for i in range(len(srcfiles)):
        dest_path = os.path.join(dest_dir, prefix + '_' + os.path.basename(srcfiles[i]))
        copyfile(srcfiles[i], dest_path)



import glob
import os
import sys
import tqdm
sys.path.insert(0,'../../../../')
from gxl_ai_utils.utils import utils_file
def handle_path(input_new_tts_final_path, only_name=False):
    """"""
    # 郭钊版：/root/path/282492425329_CPKwq_86_6299.wav
    if not only_name:
        file_name = utils_file.get_file_pure_name_from_path(input_new_tts_final_path)
    else:
        file_name = input_new_tts_final_path
    file_name_item_list = file_name.strip().split("_")
    parent_new_name = file_name_item_list[0] + "_" + file_name_item_list[1]
    if file_name_item_list[2].startswith("VAD"):
        index = int(file_name_item_list[2].split("VAD")[1])
    else:
        index = int(file_name_item_list[2])
    millisecond_num = int(file_name_item_list[3])
    return parent_new_name, index, millisecond_num

def get_little_final(input_final_scp_path, output_dir):
    """从非常大的tts_final_scp中取出一小点，组成一个测试用的小的tts_final_scp文件"""
    all_dict = utils_file.load_dict_from_scp(input_final_scp_path)
    little_dict = utils_file.get_random_subdict(all_dict, 25)
    utils_file.write_dict_to_scp(little_dict, os.path.join(output_dir, f"final_scp_little.scp"))


def little_func4get_text_1(wav_scp_dict, tts_dict, text_scp_dict:dict):
    temp_text_scp_dict = {}
    for key in tqdm.tqdm(wav_scp_dict.keys(), total=len(wav_scp_dict)):
        value = tts_dict[key]
        temp_list = value.split()
        if len(temp_list) == 2:
            wav_path = temp_list[0]
            txt_path = temp_list[1]
            line_i = utils_file.load_first_row_clean(txt_path)
            if len(line_i) > 0:
                list_i = line_i.split()
                if len(list_i) >= 2:
                    key_tmp = list_i[0]
                    value = ' '.join(list_i[1:])
                    key = key.strip()
                    value = value.strip()
                    if key == key_tmp:
                        temp_text_scp_dict[key] = value
                    else:
                        utils_file.logging_print(f'error: key不相等，key：{key}, key_tmp：{key_tmp}, value:{value}')
                else:
                    utils_file.logging_print(f'error: line_i没两个item，key：{key}, line_i：{line_i}')
        else:
            utils_file.logging_print(f'error: value不是分为wav_path 和txt_path，key：{key}, value：{value}')
    text_scp_dict.update(temp_text_scp_dict)

def post_process(output_dir,tts_final_path,shard_output_dir, text_dir=None, TTS_FINAL_TYPE=0):
    utils_file.logging_print('耿雪龙： 通用后处理函数')
    utils_file.logging_print('开始后处理')
    # 这里的tts final_path 只是处理后的wav_list了，没了text信息
    utils_file.logging_print('开始得到tts_wav_dict')
    if TTS_FINAL_TYPE == 0:
        tts_wav_path_list = utils_file.load_list_file_clean(tts_final_path)
        tts_dict = {}
        for tts_wav_path in tts_wav_path_list:
            key = utils_file.get_file_pure_name_from_path(tts_wav_path)
            tts_dict[key] = tts_wav_path
        little_dict = utils_file.get_subdict(tts_dict, 0, 10)
        utils_file.print_dict(little_dict)
    elif TTS_FINAL_TYPE == 1:
        tts_dict = utils_file.load_dict_from_scp(tts_final_path)
        little_dict = utils_file.get_subdict(tts_dict, 0, 10)
        utils_file.print_dict(little_dict)
    else:
        tts_dict = {}
    utils_file.logging_print('得到tts_wav_dict成功,总的tts_wav_dict数为：', len(tts_dict))

    utils_file.logging_print('开始得到总的切割音频的wav_dict：')
    if os.path.exists(os.path.join(output_dir, 'split_wav_big.scp')):
        split_wav_big_dict = utils_file.load_dict_from_scp(os.path.join(output_dir, 'split_wav_big.scp'))
    else:
        asr_wav_dir = os.path.join(output_dir, 'final_wav')
        split_wav_big_dict = utils_file.get_scp_for_wav_dir(asr_wav_dir)
        utils_file.write_dict_to_scp(split_wav_big_dict, os.path.join(output_dir, 'split_wav_big.scp'))
    little_split_dict = utils_file.get_subdict(split_wav_big_dict, 0, 10)
    utils_file.print_dict(little_split_dict)
    utils_file.logging_print('得到总的切割音频：完成,总的切割音频数为：', len(split_wav_big_dict))

    utils_file.logging_print('然后验证一下和tts wav的key关系')

    tmp_key = list(tts_dict.keys())[0]
    if 'VAD' in tmp_key:
        utils_file.logging_print('含有VAD的key风格,重新设计split_wav_dict的key')
        new_split_dict = {}
        for key, value in split_wav_big_dict.items():
            name, i, d = handle_path(key, only_name=True)
            duration = int(d/1000) + 1
            new_key = f"{name}_VAD{i}_{duration}"
            new_split_dict[new_key] = value
        little_dict = utils_file.get_subdict(new_split_dict, 0, 10)
        utils_file.print_dict(little_dict)
        split_wav_big_dict = new_split_dict

    num = 10
    for key, value in tts_dict.items():
        long_new_name, index, duration = handle_path(key, only_name=True)
        # key_new = f"{long_new_name}_{index}"
        if key in split_wav_big_dict:
            utils_file.logging_print('-----------')
            utils_file.logging_print(f'tts key：{key}, value：{value}')
            utils_file.logging_print(f'asr key：{key}, value：{split_wav_big_dict[key]}')
            utils_file.logging_print('-----------')
            utils_file.logging_print('\n')
            num -= 1
            if num < 0:
                break
    is_verify = num < 0
    utils_file.logging_print(f"验证一下tts wav和asr wav的key关系：验证完成,验证结果为:{is_verify}")

    utils_file.logging_print('如果验证成功，那么就计算覆盖率')
    if is_verify:
        num = 0
        for key, value in tts_dict.items():
            long_new_name, index, duration = handle_path(key, only_name=True)
            # key_new = f"{long_new_name}_{index}"
            if key in split_wav_big_dict:
                num += 1
        utils_file.logging_print(f'tts_final覆盖率为：{num / len(tts_dict)}')

    if not is_verify:
        return
    utils_file.logging_print('开始得到wav.scp')
    if os.path.exists(os.path.join(output_dir, 'wav.scp')):
        wav_scp_dict = utils_file.load_dict_from_scp(os.path.join(output_dir, 'wav.scp'))
    else:
        wav_scp_dict = {}
        for key, value in tts_dict.items():
            # long_new_name, index, duration = handle_path(key, only_name=True)
            # key_new = f"{long_new_name}_{index}"
            if key in split_wav_big_dict:
                wav_scp_dict[key] = split_wav_big_dict[key]
        utils_file.write_dict_to_scp(wav_scp_dict, os.path.join(output_dir, 'wav.scp'))
    utils_file.logging_print('得到wav.scp：完成, 长度为：', len(wav_scp_dict))

    utils_file.logging_print(f'开始得到text.scp， TTS_FINAL_TYPE：{TTS_FINAL_TYPE}')
    text_scp_dict = {}
    if os.path.exists(os.path.join(output_dir, 'text_with_punctuation')):
        utils_file.logging_print('text_with_punctuation已存在，不用做任何事')
    else:
        if TTS_FINAL_TYPE == 0:
            temp_txt_file_dict = {}
            txt_file_list = glob.glob(str(os.path.join(text_dir, '*.txt')))
            for txt_file in tqdm.tqdm(txt_file_list, total=len(txt_file_list)):
                key = utils_file.get_file_pure_name_from_path(txt_file)
                temp_txt_file_dict[key] = txt_file
            for key in tqdm.tqdm(wav_scp_dict.keys(), total=len(wav_scp_dict)):
                if key in temp_txt_file_dict:
                    txt_file = temp_txt_file_dict[key]
                    line_i = utils_file.load_first_row_clean(txt_file)
                    if len(line_i) > 0:
                        list_i = line_i.split()
                        if len(list_i) >= 2:
                            key_tmp = list_i[0]
                            value = ' '.join(list_i[1:])
                            key = key.strip()
                            value = value.strip()
                            if key == key_tmp:
                                text_scp_dict[key] = value
                            else:
                                utils_file.logging_print(f'error: key不相等，key：{key}, key_tmp：{key_tmp}, value:{value}')
                        else:
                            utils_file.logging_print(f'error: line_i没两个item，key：{key}, line_i：{line_i}')
        elif TTS_FINAL_TYPE == 1:
            num_thread = 20
            runner = utils_file.GxlDynamicThreadPool()
            wav_scp_dict_list = utils_file.do_split_dict(wav_scp_dict, num_thread)
            for wav_scp_dict_i in wav_scp_dict_list:
                runner.add_thread(little_func4get_text_1, [wav_scp_dict_i,tts_dict, text_scp_dict])
            runner.start()
        utils_file.write_dict_to_scp(text_scp_dict, os.path.join(output_dir, 'text_with_punctuation'))
        utils_file.logging_print(f'text_scp得到完成,text_scp_dict长度为：{len(text_scp_dict)}')

    utils_file.logging_print('开始为text清除标点')
    if os.path.exists( os.path.join(output_dir, 'text')):
        utils_file.logging_print('text已存在，不用做任何事')
        pass
    else:
        text_with_punctuation_dict = utils_file.load_dict_from_scp(os.path.join(output_dir, 'text_with_punctuation'))
        text_no_punctuation_dict = {}
        for key, value in text_with_punctuation_dict.items():
            text_no_punctuation_dict[key] = utils_file.do_filter(value)
        utils_file.write_dict_to_scp(text_no_punctuation_dict, os.path.join(output_dir, 'text'))
    utils_file.logging_print('清除标点完成,len: ', len(text_no_punctuation_dict))

    utils_file.logging_print('开始打shard包')
    utils_file.do_make_shard_file(os.path.join(output_dir, "wav.scp"), os.path.join(output_dir, "text"),
                                  shard_output_dir, num_threads=50)
    utils_file.logging_print('打shard包完成')




#!/usr/bin/env python3
# Copyright    2021  Xiaomi Corp.        (authors: Fangjun Kuang)
#
# See ../../../../LICENSE for clarification regarding multiple authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Usage:
    ./rnn_lm/train.py \
        --start-epoch 0 \
        --world-size 2 \
        --num-epochs 1 \
        --use-fp16 0 \
        --tie-weights 0 \
        --embedding-dim 800 \
        --hidden-dim 200 \
        --num-layers 2 \
        --batch-size 400

"""

import argparse
import logging
import math
from pathlib import Path
from shutil import copyfile
from typing import Optional, Tuple

import torch
import torch.multiprocessing as mp
import torch.nn as nn
import torch.optim as optim
from dataset import get_dataloader
from lhotse.utils import fix_random_seed
from model import RnnLmModel
from torch.nn.parallel import DistributedDataParallel as DDP
from torch.nn.utils import clip_grad_norm_
from torch.utils.tensorboard import SummaryWriter

from icefall.checkpoint import load_checkpoint
from icefall.checkpoint import save_checkpoint as save_checkpoint_impl
from icefall.checkpoint import save_checkpoint_with_global_batch_idx
from icefall.dist import cleanup_dist, setup_dist
from icefall.env import get_env_info
from icefall.utils import AttributeDict, MetricsTracker, setup_logger, str2bool


def get_parser():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )

    parser.add_argument(
        "--world-size",
        type=int,
        default=1,
        help="Number of GPUs for DDP training.",
    )

    parser.add_argument(
        "--master-port",
        type=int,
        default=12354,
        help="Master port to use for DDP training.",
    )

    parser.add_argument(
        "--tensorboard",
        type=str2bool,
        default=True,
        help="Should various information be logged in tensorboard.",
    )

    parser.add_argument(
        "--num-epochs",
        type=int,
        default=30,
        help="Number of epochs to train.",
    )

    parser.add_argument(
        "--start-epoch",
        type=int,
        default=0,
        help="""Resume training from from this epoch.
        If it is positive, it will load checkpoint from
        exp_dir/epoch-{start_epoch-1}.pt
        """,
    )

    parser.add_argument(
        "--start-batch",
        type=int,
        default=0,
        help="""If positive, --start-epoch is ignored and
        it loads the checkpoint from exp-dir/checkpoint-{start_batch}.pt
        """,
    )

    parser.add_argument(
        "--exp-dir",
        type=str,
        default="rnn_lm/exp",
        help="""The experiment dir.
        It specifies the directory where all training related
        files, e.g., checkpoints, logs, etc, are saved
        """,
    )

    parser.add_argument(
        "--use-fp16",
        type=str2bool,
        default=True,
        help="Whether to use half precision training.",
    )

    parser.add_argument(
        "--batch-size",
        type=int,
        default=400,
    )

    parser.add_argument(
        "--lm-data",
        type=str,
        default="data/lm_training_bpe_500/sorted_lm_data.pt",
        help="LM training data",
    )

    parser.add_argument(
        "--lm-data-valid",
        type=str,
        default="data/lm_training_bpe_500/sorted_lm_data-valid.pt",
        help="LM validation data",
    )

    parser.add_argument(
        "--vocab-size",
        type=int,
        default=500,
        help="Vocabulary size of the model",
    )

    parser.add_argument(
        "--embedding-dim",
        type=int,
        default=2048,
        help="Embedding dim of the model",
    )

    parser.add_argument(
        "--hidden-dim",
        type=int,
        default=2048,
        help="Hidden dim of the model",
    )

    parser.add_argument(
        "--num-layers",
        type=int,
        default=3,
        help="Number of RNN layers the model",
    )

    parser.add_argument(
        "--tie-weights",
        type=str2bool,
        default=True,
        help="""True to share the weights between the input embedding layer and the
        last output linear layer
        """,
    )

    parser.add_argument(
        "--seed",
        type=int,
        default=42,
        help="The seed for random generators intended for reproducibility",
    )

    parser.add_argument(
        "--lr",
        type=float,
        default=1e-3,
    )

    parser.add_argument(
        "--max-sent-len",
        type=int,
        default=200,
        help="""Maximum number of tokens in a sentence. This is used
        to adjust batch-size dynamically""",
    )

    parser.add_argument(
        "--save-every-n",
        type=int,
        default=2000,
        help="""Save checkpoint after processing this number of batches"
        periodically. We save checkpoint to exp-dir/ whenever
        params.batch_idx_train % save_every_n == 0. The checkpoint filename
        has the form: f'exp-dir/checkpoint-{params.batch_idx_train}.pt'
        Note: It also saves checkpoint to `exp-dir/epoch-xxx.pt` at the
        end of each epoch where `xxx` is the epoch number counting from 0.
        """,
    )

    return parser


def get_params() -> AttributeDict:
    """Return a dict containing training parameters."""

    params = AttributeDict(
        {
            "max_sent_len": 200,
            "sos_id": 1,
            "eos_id": 1,
            "blank_id": 0,
            "weight_decay": 1e-6,
            "best_train_loss": float("inf"),
            "best_valid_loss": float("inf"),
            "best_train_epoch": -1,
            "best_valid_epoch": -1,
            "batch_idx_train": 0,
            "log_interval": 100,
            "reset_interval": 2000,
            "valid_interval": 200,
            "env_info": get_env_info(),
        }
    )
    return params


def load_checkpoint_if_available(
    params: AttributeDict,
    model: nn.Module,
    optimizer: Optional[torch.optim.Optimizer] = None,
    scheduler: Optional[torch.optim.lr_scheduler._LRScheduler] = None,
) -> None:
    """Load checkpoint from file.

    If params.start_batch is positive, it will load the checkpoint from
    `params.exp_dir/checkpoint-{params.start_batch}.pt`. Otherwise, if
    params.start_epoch is larger than 1, it will load the checkpoint from
    `params.start_epoch - 1`. Otherwise, this function does nothing.

    Apart from loading state dict for `model`, `optimizer` and `scheduler`,
    it also updates `best_train_epoch`, `best_train_loss`, `best_valid_epoch`,
    and `best_valid_loss` in `params`.

    Args:
      params:
        The return value of :func:`get_params`.
      model:
        The training model.
      optimizer:
        The optimizer that we are using.
      scheduler:
        The learning rate scheduler we are using.
    Returns:
      Return None.
    """

    if params.start_batch > 0:
        filename = params.exp_dir / f"checkpoint-{params.start_batch}.pt"
    elif params.start_epoch > 1:
        filename = params.exp_dir / f"epoch-{params.start_epoch-1}.pt"
    else:
        return None

    logging.info(f"Loading checkpoint: {filename}")
    saved_params = load_checkpoint(
        filename,
        model=model,
        optimizer=optimizer,
        scheduler=scheduler,
    )

    keys = [
        "best_train_epoch",
        "best_valid_epoch",
        "batch_idx_train",
        "best_train_loss",
        "best_valid_loss",
    ]
    for k in keys:
        params[k] = saved_params[k]

    if params.start_batch > 0:
        if "cur_epoch" in saved_params:
            params["start_epoch"] = saved_params["cur_epoch"]

        if "cur_batch_idx" in saved_params:
            params["cur_batch_idx"] = saved_params["cur_batch_idx"]

    return saved_params


def save_checkpoint(
    params: AttributeDict,
    model: nn.Module,
    optimizer: Optional[torch.optim.Optimizer] = None,
    scheduler: Optional[torch.optim.lr_scheduler._LRScheduler] = None,
    rank: int = 0,
) -> None:
    """Save model, optimizer, scheduler and training stats to file.

    Args:
      params:
        It is returned by :func:`get_params`.
      model:
        The training model.
    """
    if rank != 0:
        return
    filename = params.exp_dir / f"epoch-{params.cur_epoch}.pt"
    save_checkpoint_impl(
        filename=filename,
        model=model,
        params=params,
        optimizer=optimizer,
        scheduler=scheduler,
        rank=rank,
    )

    if params.best_train_epoch == params.cur_epoch:
        best_train_filename = params.exp_dir / "best-train-loss.pt"
        copyfile(src=filename, dst=best_train_filename)

    if params.best_valid_epoch == params.cur_epoch:
        best_valid_filename = params.exp_dir / "best-valid-loss.pt"
        copyfile(src=filename, dst=best_valid_filename)


def compute_loss(
    model: nn.Module,
    x: torch.Tensor,
    y: torch.Tensor,
    sentence_lengths: torch.Tensor,
    is_training: bool,
) -> Tuple[torch.Tensor, MetricsTracker]:
    """Compute the negative log-likelihood loss given a model and its input.
    Args:
      model:
        The NN model, e.g., RnnLmModel.
      x:
        A 2-D tensor. Each row contains BPE token IDs for a sentence. Also,
        each row starts with SOS ID.
      y:
        A 2-D tensor. Each row is a shifted version of the corresponding row
        in `x` but ends with an EOS ID (before padding).
     sentence_lengths:
       A 1-D tensor containing number of tokens of each sentence
       before padding.
     is_training:
       True for training. False for validation.
    """
    with torch.set_grad_enabled(is_training):
        device = model.device
        x = x.to(device)
        y = y.to(device)
        sentence_lengths = sentence_lengths.to(device)

        nll = model(x, y, sentence_lengths)
        loss = nll.sum()

        num_tokens = sentence_lengths.sum().item()

        loss_info = MetricsTracker()
        # Note: Due to how MetricsTracker() is designed,
        # we use "frames" instead of "num_tokens" as a key here
        loss_info["frames"] = num_tokens
        loss_info["loss"] = loss.detach().item()
    return loss, loss_info


def compute_validation_loss(
    params: AttributeDict,
    model: nn.Module,
    valid_dl: torch.utils.data.DataLoader,
    world_size: int = 1,
) -> MetricsTracker:
    """Run the validation process. The validation loss
    is saved in `params.valid_loss`.
    """
    model.eval()

    tot_loss = MetricsTracker()

    for batch_idx, batch in enumerate(valid_dl):
        x, y, sentence_lengths = batch
        with torch.cuda.amp.autocast(enabled=params.use_fp16):
            loss, loss_info = compute_loss(
                model=model,
                x=x,
                y=y,
                sentence_lengths=sentence_lengths,
                is_training=False,
            )

        assert loss.requires_grad is False
        tot_loss = tot_loss + loss_info

    if world_size > 1:
        tot_loss.reduce(loss.device)

    loss_value = tot_loss["loss"] / tot_loss["frames"]
    if loss_value < params.best_valid_loss:
        params.best_valid_epoch = params.cur_epoch
        params.best_valid_loss = loss_value

    return tot_loss


def train_one_epoch(
    params: AttributeDict,
    model: nn.Module,
    optimizer: torch.optim.Optimizer,
    train_dl: torch.utils.data.DataLoader,
    valid_dl: torch.utils.data.DataLoader,
    tb_writer: Optional[SummaryWriter] = None,
    world_size: int = 1,
    rank: int = 0,
) -> None:
    """Train the model for one epoch.

    The training loss from the mean of all sentences is saved in
    `params.train_loss`. It runs the validation process every
    `params.valid_interval` batches.

    Args:
      params:
        It is returned by :func:`get_params`.
      model:
        The model for training.
      optimizer:
        The optimizer we are using.
      train_dl:
        Dataloader for the training dataset.
      valid_dl:
        Dataloader for the validation dataset.
      tb_writer:
        Writer to write log messages to tensorboard.
      world_size:
        Number of nodes in DDP training. If it is 1, DDP is disabled.
    """
    model.train()

    tot_loss = MetricsTracker()

    cur_batch_idx = params.get("cur_batch_idx", 0)

    for batch_idx, batch in enumerate(train_dl):
        if batch_idx < cur_batch_idx:
            continue
        cur_batch_idx = batch_idx

        params.batch_idx_train += 1
        x, y, sentence_lengths = batch
        batch_size = x.size(0)
        with torch.cuda.amp.autocast(enabled=params.use_fp16):
            loss, loss_info = compute_loss(
                model=model,
                x=x,
                y=y,
                sentence_lengths=sentence_lengths,
                is_training=True,
            )

        # summary stats
        tot_loss = (tot_loss * (1 - 1 / params.reset_interval)) + loss_info

        optimizer.zero_grad()
        loss.backward()
        clip_grad_norm_(model.parameters(), 5.0, 2.0)
        optimizer.step()

        if (
            params.batch_idx_train > 0
            and params.batch_idx_train % params.save_every_n == 0
        ):
            params.cur_batch_idx = batch_idx
            save_checkpoint_with_global_batch_idx(
                out_dir=params.exp_dir,
                global_batch_idx=params.batch_idx_train,
                model=model,
                params=params,
                optimizer=optimizer,
                rank=rank,
            )
            del params.cur_batch_idx

        if batch_idx % params.log_interval == 0:
            # Note: "frames" here means "num_tokens"
            this_batch_ppl = math.exp(loss_info["loss"] / loss_info["frames"])
            tot_ppl = math.exp(tot_loss["loss"] / tot_loss["frames"])

            logging.info(
                f"Epoch {params.cur_epoch}, "
                f"batch {batch_idx}, loss[{loss_info}, ppl: {this_batch_ppl}] "
                f"tot_loss[{tot_loss}, ppl: {tot_ppl}], "
                f"batch size: {batch_size}"
            )

            if tb_writer is not None:
                loss_info.write_summary(
                    tb_writer, "train/current_", params.batch_idx_train
                )
                tot_loss.write_summary(tb_writer, "train/tot_", params.batch_idx_train)

                tb_writer.add_scalar(
                    "train/current_ppl", this_batch_ppl, params.batch_idx_train
                )

                tb_writer.add_scalar("train/tot_ppl", tot_ppl, params.batch_idx_train)

        if batch_idx > 0 and batch_idx % params.valid_interval == 0:
            logging.info("Computing validation loss")

            valid_info = compute_validation_loss(
                params=params,
                model=model,
                valid_dl=valid_dl,
                world_size=world_size,
            )
            model.train()

            valid_ppl = math.exp(valid_info["loss"] / valid_info["frames"])
            logging.info(
                f"Epoch {params.cur_epoch}, validation: {valid_info}, "
                f"ppl: {valid_ppl}"
            )

            if tb_writer is not None:
                valid_info.write_summary(
                    tb_writer, "train/valid_", params.batch_idx_train
                )

                tb_writer.add_scalar(
                    "train/valid_ppl", valid_ppl, params.batch_idx_train
                )

    loss_value = tot_loss["loss"] / tot_loss["frames"]
    params.train_loss = loss_value
    if params.train_loss < params.best_train_loss:
        params.best_train_epoch = params.cur_epoch
        params.best_train_loss = params.train_loss


def run(rank, world_size, args):
    """
    Args:
      rank:
        It is a value between 0 and `world_size-1`, which is
        passed automatically by `mp.spawn()` in :func:`main`.
        The node with rank 0 is responsible for saving checkpoint.
      world_size:
        Number of GPUs for DDP training.
      args:
        The return value of get_parser().parse_args()
    """
    params = get_params()
    params.update(vars(args))
    is_distributed = world_size > 1

    fix_random_seed(params.seed)
    if is_distributed:
        setup_dist(rank, world_size, params.master_port)

    setup_logger(f"{params.exp_dir}/log/log-train")
    logging.info("Training started")
    logging.info(params)

    if args.tensorboard and rank == 0:
        tb_writer = SummaryWriter(log_dir=f"{params.exp_dir}/tensorboard")
    else:
        tb_writer = None

    device = torch.device("cpu")
    if torch.cuda.is_available():
        device = torch.device("cuda", rank)

    logging.info(f"Device: {device}")

    logging.info("About to create model")
    model = RnnLmModel(
        vocab_size=params.vocab_size,
        embedding_dim=params.embedding_dim,
        hidden_dim=params.hidden_dim,
        num_layers=params.num_layers,
        tie_weights=params.tie_weights,
    )

    num_param = sum([p.numel() for p in model.parameters()])
    logging.info(f"Number of model parameters: {num_param}")

    checkpoints = load_checkpoint_if_available(params=params, model=model)

    model.to(device)
    if is_distributed:
        model = DDP(model, device_ids=[rank])

    model.device = device

    optimizer = optim.Adam(
        model.parameters(),
        lr=params.lr,
        weight_decay=params.weight_decay,
    )
    if checkpoints:
        logging.info("Load optimizer state_dict from checkpoint")
        optimizer.load_state_dict(checkpoints["optimizer"])

    logging.info(f"Loading LM training data from {params.lm_data}")
    train_dl = get_dataloader(
        filename=params.lm_data,
        is_distributed=is_distributed,
        params=params,
    )

    logging.info(f"Loading LM validation data from {params.lm_data_valid}")
    valid_dl = get_dataloader(
        filename=params.lm_data_valid,
        is_distributed=is_distributed,
        params=params,
    )

    # Note: No learning rate scheduler is used here
    for epoch in range(params.start_epoch, params.num_epochs):
        if is_distributed:
            train_dl.sampler.set_epoch(epoch)

        params.cur_epoch = epoch

        train_one_epoch(
            params=params,
            model=model,
            optimizer=optimizer,
            train_dl=train_dl,
            valid_dl=valid_dl,
            tb_writer=tb_writer,
            world_size=world_size,
            rank=rank,
        )

        save_checkpoint(
            params=params,
            model=model,
            optimizer=optimizer,
            rank=rank,
        )

    logging.info("Done!")

    if is_distributed:
        torch.distributed.barrier()
        cleanup_dist()


def main():
    parser = get_parser()
    args = parser.parse_args()
    args.exp_dir = Path(args.exp_dir)

    world_size = args.world_size
    assert world_size >= 1
    if world_size > 1:
        mp.spawn(run, args=(world_size, args), nprocs=world_size, join=True)
    else:
        run(rank=0, world_size=1, args=args)


torch.set_num_threads(1)
torch.set_num_interop_threads(1)

if __name__ == "__main__":
    main()



import codecs
import logging
import subprocess
from concurrent.futures import ThreadPoolExecutor
import json
import os

def load_list_file_clean(path: str):
    """
    得到不包含换行符的str_list
    :param path:
    :return:
    """
    if not os.path.exists(path):
        print(f'load_list_file_clean()_{path}文件不存在')
        return []

    with codecs.open(path, 'r', encoding='utf=8') as f:
        cat_to_name: list = f.read().splitlines()
        # cat_to_name: list = f.readlines() -> 包含换行符
        print(f"load_list_file_clean()_数据总条数为:{len(cat_to_name)}")
    return cat_to_name


# # 假设这是你的YouTube视频URL列表
# urls = [
#     "https://www.youtube.com/watch?v=Ds6rTnXZqUg",
#     # 添加更多的URLs
# ]

# 初始化日志列表，用于存储失败的下载索引
log = {'downed_num': 0, 'err': []}


def download_audio_as_wav(url, output_template, index):
    command = [
        'yt-dlp',
        '-x',  # 仅提取音频
        '--audio-format', 'mp3',  # 设置音频格式为wav
        '--output', output_template,  # 设置输出文件名模板
        url  # YouTube视频URL
    ]
    result = subprocess.run(command, capture_output=True, text=True)
    # 如果返回代码不是0（即发生错误），则记录索引
    if result.returncode != 0:
        log['err'].append({
            "index": index,
            "url": url,
            "error": result.stderr
        })


# 使用ThreadPoolExecutor来管理多线程
def download_videos(urls, pre_num):
    with ThreadPoolExecutor(max_workers=5) as executor:
        for index, url in enumerate(urls):
            output_template = f"{index + pre_num:010d}.%(ext)s"
            executor.submit(download_audio_as_wav, url, output_template, index + pre_num)





if os.path.exists('./log.json'):
    with open('./log.json', 'r') as js:
        log = json.load(js)

urls = []
urls = load_list_file_clean(
    '/home/work_nfs8/xlgeng/new_workspace/gxl_ai_utils/eggs/spider/youtube_dl/B_true.txt')
print('共有' + str(len(urls)) + '个视频')
my_utils = urls[:164]
print('我处理' + str(len(my_utils)) + '个视频')
output_dir = '/home/work_nfs14/xlgeng/youtube_data'
# os.makedirs(output_dir)
urls = []
for video_path in my_utils:
    urls.append(video_path.strip())
    print('把' + video_path + '加入队列')
    if len(urls) == 10:
        print('开始下载')
        download_videos(urls, log['downed_num'])
        urls = []
        log['downed_num'] = log['downed_num'] + 10
        print('has downed' + str(log['downed_num']) + 'lines')
        with open('log.json', 'w') as log_file:
            json.dump(log, log_file, indent=4)
        break


# with open('/home/node36_data/kxxia/song/DISCO-10M_train_noembed.csv', 'r') as file:
#     # head  = file.readline()
#     pre_num = log['downed_num']
#     for i in range(pre_num):
#         _ = file.readline()
#     for line in file:
#         urls.append(line.strip().split(',')[0])
#         if len(urls) == 10:
#             download_videos(urls, log['downed_num'])
#             urls = []
#             log['downed_num'] = log['downed_num'] + 10
#             print('has downed' + str(log['downed_num']) + 'lines')
#             with open('log.json', 'w') as log_file:
#                 json.dump(log, log_file, indent=4)



# Copyright (c) 2020 Mobvoi Inc (Binbin Zhang)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
import logging
from contextlib import nullcontext

# if your python version < 3.7 use the below one
# from contextlib import suppress as nullcontext
import torch

from ..utils.train_utils import (wenet_join, batch_forward, batch_backward,
                                     update_parameter_and_lr, log_per_step)


class Executor:

    def __init__(self):
        self.step = 0

    def train(self, model, optimizer, scheduler, data_loader, writer,
              configs, scaler, group_join):
        ''' Train one epoch
        '''
        model.train()
        info_dict = copy.deepcopy(configs)
        info_dict["tag"] = "TRAIN"
        logging.info('using accumulate grad, new batch size is {} times'
                     ' larger than before'.format(info_dict['accum_grad']))

        # A context manager to be used in conjunction with an instance of
        # torch.nn.parallel.DistributedDataParallel to be able to train
        # with uneven inputs across participating processes.
        if isinstance(model, torch.nn.parallel.DistributedDataParallel):
            model_context = model.join
        else:
            model_context = nullcontext

        with model_context():
            for batch_idx, batch_dict in enumerate(data_loader):
                info_dict["step"] = self.step
                info_dict["batch_idx"] = batch_idx
                if wenet_join(group_join, info_dict):
                    break

                if batch_dict["target_lengths"].size(0) == 0:
                    continue

                context = None
                # Disable gradient synchronizations across DDP processes.
                # Within this context, gradients will be accumulated on module
                # variables, which will later be synchronized.
                if info_dict.get("train_engine", "torch_ddp") == "torch_ddp" and \
                        (batch_idx + 1) % info_dict["accum_grad"] != 0 and torch.cuda.is_available():
                    context = model.no_sync
                # Used for single gpu training and DDP gradient synchronization
                # processes.
                else:
                    context = nullcontext

                with context():
                    info_dict = batch_forward(model, batch_dict, scaler, info_dict)
                    info_dict = batch_backward(model, scaler, info_dict)

                info_dict = update_parameter_and_lr(
                    model, optimizer, scheduler,
                    scaler, info_dict
                )
                log_per_step(writer, info_dict)
                self.step += 1

    def cv(self, model, data_loader, configs):
        ''' Cross validation on
        '''
        model.eval()
        info_dict = copy.deepcopy(configs)
        info_dict["tag"] = "CV"
        num_seen_utts, total_loss = 1, 0.0  # in order to avoid division by 0
        with torch.no_grad():
            for batch_idx, batch_dict in enumerate(data_loader):
                info_dict["step"] = self.step
                info_dict["batch_idx"] = batch_idx

                num_utts = batch_dict["target_lengths"].size(0)
                if num_utts == 0:
                    continue

                info_dict = batch_forward(model, batch_dict, None, info_dict)
                loss = info_dict['loss_dict']['loss']

                if torch.isfinite(loss):
                    num_seen_utts += num_utts
                    total_loss += loss.item() * num_utts

                info_dict["history_loss"] = total_loss / num_seen_utts
                log_per_step(writer=None, info_dict=info_dict)
        return total_loss, num_seen_utts



import torch


class RMSNorm(torch.nn.Module):
    """ https://arxiv.org/pdf/1910.07467.pdf
    """

    def __init__(
        self,
        dim: int,
        eps: float = 1e-6,
    ):
        super().__init__()
        self.eps = eps
        self.weight = torch.nn.Parameter(torch.ones(dim))

    def _norm(self, x):
        return x * torch.rsqrt(x.pow(2).mean(-1, keepdim=True) + self.eps)

    def forward(self, x):
        x = self._norm(x.float()).type_as(x)
        return x * self.weight



from torch import nn
from .scaled_dot_product_attention import ScaledDotProductAttention


class MultiHeadAttention(nn.Module):
    def __init__(self, model_dim=512, num_heads=8, dropout=0.0):
        super(MultiHeadAttention, self).__init__()
        self.dim_per_head = model_dim // num_heads
        self.num_heads = num_heads

        self.query = nn.Linear(model_dim, model_dim)
        self.key = nn.Linear(model_dim, model_dim)
        self.value = nn.Linear(model_dim, model_dim)
        self.out = nn.Linear(model_dim, model_dim)

        self.attn = ScaledDotProductAttention(dropout)

        self.dropout = nn.Dropout(dropout)
        self.layer_norm = nn.LayerNorm(model_dim)

    def forward(self, query, key, value, attn_mask=None):
        """
        query: [batch_size, query_len, model_dim]
        key: [batch_size, key_len, model_dim]
        value: [batch_size, value_len, model_dim]
        attn_mask: [batch_size, query_len, key_len]
        """
        residual = query
        batch_size = query.size(0)
        num_heads = self.num_heads
        dim_per_head = self.dim_per_head

        query = self.query(query)
        key = self.key(key)
        value = self.value(value)
        query = query.view(batch_size, -1, num_heads, dim_per_head).transpose(1, 2)
        key = key.view(batch_size, -1, num_heads, dim_per_head).transpose(1, 2)
        value = value.view(batch_size, -1, num_heads, dim_per_head).transpose(1, 2)
        if attn_mask:
            attn_mask.repeat(1, num_heads, 1, 1)
        scale = key.size(-1) ** -0.5
        context, attention = self.attn(query, key, value, scale, attn_mask)
        context = context.transpose(1, 2).contiguous().view(batch_size, -1, num_heads * dim_per_head)
        output = self.out(context)
        output = self.dropout(output) + residual
        output = self.layer_norm(output)
        return output, attention



import os
import re

import tqdm
import sys

sys.path.insert(0, '../../../')
from gxl_ai_utils.utils import utils_file


def do_remove_punctuation(text):
    # 使用正则表达式去除标点符号，只保留汉字、英文和数字
    return re.sub(r'[^\u4e00-\u9fa5a-zA-Z0-9]', '', text)


def make_shard():
    """"""
    target_dir = "/home/work_nfs14/xlgeng/asr_data_shard/pachong_data"
    dataname_list = ["ximalaya_lishi_10T-1", "ximalaya_lishi_10T", "ximalaya_redian_2T"]
    utils_file.logging_print('开始清理text, 清理完毕')
    utils_file.logging_print('开始合并')
    all_list = []
    for dataname in dataname_list:
        temp_dir = os.path.join(target_dir, dataname)
        shards_list_path = os.path.join(temp_dir, "shards_list.txt")
        temp_shard_list = utils_file.load_list_file_clean(shards_list_path)
        all_list.extend(temp_shard_list)
    utils_file.write_list_to_file(all_list, os.path.join(target_dir, "all_shards_list.txt"))
if __name__ == '__main__':
    make_shard()



# Copyright (c)  2021  Xiaomi Corporation (authors: Daniel Povey, Fangjun Kuang)
#
# See ../../../../LICENSE for clarification regarding multiple authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import List, Tuple

import k2
import torch
from torch.utils.data import DataLoader
from torch.utils.data.distributed import DistributedSampler

from icefall.utils import AttributeDict, add_eos, add_sos


class LmDataset(torch.utils.data.Dataset):
    def __init__(
        self,
        sentences: k2.RaggedTensor,
        words: k2.RaggedTensor,
        sentence_lengths: torch.Tensor,
        max_sent_len: int,
        batch_size: int,
    ):
        """
        Args:
          sentences:
            A ragged tensor of dtype torch.int32 with 2 axes [sentence][word].
          words:
            A ragged tensor of dtype torch.int32 with 2 axes [word][token].
          sentence_lengths:
            A 1-D tensor of dtype torch.int32 containing number of tokens
            of each sentence.
          max_sent_len:
            Maximum sentence length. It is used to change the batch size
            dynamically. In general, we try to keep the product of
            "max_sent_len in a batch" and "num_of_sent in a batch" being
            a constant.
          batch_size:
            The expected batch size. It is changed dynamically according
            to the "max_sent_len".

        See `../local/prepare_lm_training_data.py` for how `sentences` and
        `words` are generated. We assume that `sentences` are sorted by length.
        See `../local/sort_lm_training_data.py`.
        """
        super().__init__()
        self.sentences = sentences
        self.words = words

        sentence_lengths = sentence_lengths.tolist()

        assert batch_size > 0, batch_size
        assert max_sent_len > 1, max_sent_len
        batch_indexes = []
        num_sentences = sentences.dim0
        cur = 0
        while cur < num_sentences:
            sz = sentence_lengths[cur] // max_sent_len + 1
            # Assume the current sentence has 3 * max_sent_len tokens,
            # in the worst case, the subsequent sentences also have
            # this number of tokens, we should reduce the batch size
            # so that this batch will not contain too many tokens
            actual_batch_size = batch_size // sz + 1
            actual_batch_size = min(actual_batch_size, batch_size)
            end = cur + actual_batch_size
            end = min(end, num_sentences)
            this_batch_indexes = torch.arange(cur, end).tolist()
            batch_indexes.append(this_batch_indexes)
            cur = end
        assert batch_indexes[-1][-1] == num_sentences - 1

        self.batch_indexes = k2.RaggedTensor(batch_indexes)

    def __len__(self) -> int:
        """Return number of batches in this dataset"""
        return self.batch_indexes.dim0

    def __getitem__(self, i: int) -> k2.RaggedTensor:
        """Get the i'th batch in this dataset
        Return a ragged tensor with 2 axes [sentence][token].
        """
        assert 0 <= i < len(self), i

        # indexes is a 1-D tensor containing sentence indexes
        indexes = self.batch_indexes[i]

        # sentence_words is a ragged tensor with 2 axes
        # [sentence][word]
        sentence_words = self.sentences[indexes]

        # in case indexes contains only 1 entry, the returned
        # sentence_words is a 1-D tensor, we have to convert
        # it to a ragged tensor
        if isinstance(sentence_words, torch.Tensor):
            sentence_words = k2.RaggedTensor(sentence_words.unsqueeze(0))

        # sentence_word_tokens is a ragged tensor with 3 axes
        # [sentence][word][token]
        sentence_word_tokens = self.words.index(sentence_words)
        assert sentence_word_tokens.num_axes == 3

        sentence_tokens = sentence_word_tokens.remove_axis(1)
        return sentence_tokens


class LmDatasetCollate:
    def __init__(self, sos_id: int, eos_id: int, blank_id: int):
        """
        Args:
          sos_id:
            Token ID of the SOS symbol.
          eos_id:
            Token ID of the EOS symbol.
          blank_id:
            Token ID of the blank symbol.
        """
        self.sos_id = sos_id
        self.eos_id = eos_id
        self.blank_id = blank_id

    def __call__(
        self, batch: List[k2.RaggedTensor]
    ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor]:
        """Return a tuple containing 3 tensors:

        - x, a 2-D tensor of dtype torch.int32; each row contains tokens
             for a sentence starting with `self.sos_id`. It is padded to
             the max sentence length with `self.blank_id`.

        - y, a 2-D tensor of dtype torch.int32; each row contains tokens
             for a sentence ending with `self.eos_id` before padding.
             Then it is padded to the max sentence length with
             `self.blank_id`.

        - lengths, a 2-D tensor of dtype torch.int32, containing the number of
                   tokens of each sentence before padding.
        """
        # The batching stuff has already been done in LmDataset
        assert len(batch) == 1
        sentence_tokens = batch[0]
        row_splits = sentence_tokens.shape.row_splits(1)
        sentence_token_lengths = row_splits[1:] - row_splits[:-1]
        sentence_tokens_with_sos = add_sos(sentence_tokens, self.sos_id)
        sentence_tokens_with_eos = add_eos(sentence_tokens, self.eos_id)

        x = sentence_tokens_with_sos.pad(mode="constant", padding_value=self.blank_id)
        y = sentence_tokens_with_eos.pad(mode="constant", padding_value=self.blank_id)
        sentence_token_lengths += 1  # plus 1 since we added a SOS

        return x.to(torch.int64), y.to(torch.int64), sentence_token_lengths


def get_dataloader(
    filename: str,
    is_distributed: bool,
    params: AttributeDict,
) -> torch.utils.data.DataLoader:
    """Get dataloader for LM training.

    Args:
      filename:
        Path to the file containing LM data. The file is assumed to
        be generated by `../local/sort_lm_training_data.py`.
      is_distributed:
        True if using DDP training. False otherwise.
      params:
        Set `get_params()` from `rnn_lm/train.py`
    Returns:
      Return a dataloader containing the LM data.
    """
    lm_data = torch.load(filename)

    words = lm_data["words"]
    sentences = lm_data["sentences"]
    sentence_lengths = lm_data["sentence_lengths"]

    dataset = LmDataset(
        sentences=sentences,
        words=words,
        sentence_lengths=sentence_lengths,
        max_sent_len=params.max_sent_len,
        batch_size=params.batch_size,
    )
    if is_distributed:
        sampler = DistributedSampler(dataset, shuffle=True, drop_last=True)
    else:
        sampler = None

    collate_fn = LmDatasetCollate(
        sos_id=params.sos_id,
        eos_id=params.eos_id,
        blank_id=params.blank_id,
    )

    dataloader = DataLoader(
        dataset,
        batch_size=1,
        collate_fn=collate_fn,
        sampler=sampler,
        shuffle=sampler is None,
    )
    return dataloader



import random

import torch.utils.data
from torch.utils.data import IterableDataset
import torch.distributed as dist
from gxl_ai_utils.utils import utils_file
import gxl_ai_utils.training.dataset.processor as processor


class Processor(IterableDataset):
    def __init__(self, source, f, *args, **kwargs):
        self.source = source
        self.f = f
        self.args = args
        self.kwargs = kwargs

    def set_epoch(self, epoch):
        self.source.set_epoch(epoch)

    def __iter__(self):
        return self.f(iter(self.source), *self.args, **self.kwargs)




class DistributedSampler:
    def __init__(self, shuffle, partition):
        self.epoch = -1
        self.update()
        self.shuffle = shuffle
        self.partition = partition
    def update(self):
        assert dist.is_available()
        if dist.is_initialized():
            self.rank = dist.get_rank()
            self.world_size = dist.get_world_size()
        else:
            self.rank = 0
            self.world_size = 1

        work_info = torch.utils.data.get_worker_info()
        if work_info is None:
            self.worker_id = 0
            self.num_workers = 1
        else:
            self.worker_id = work_info.id
            self.num_workers = work_info.num_workers
        return dict(
            rank = self.rank,
            world_size = self.world_size,
            worker_id = self.worker_id,
            num_workers = self.num_workers
        )

    def set_epoch(self, epoch):
        self.epoch = epoch

    def sample(self, data):
        """
        根据worker_id ,rank进行sample
        :param data:
        :return:
        """
        data = list(range(len(data)))
        if self.shuffle:
            random.Random(self.epoch).shuffle(data)
        if self.partition:
            data = data[self.rank::self.world_size]
        data = data[self.worker_id::self.num_workers]
        return data



class DataList(IterableDataset):
    def __init__(self,
                 lists,
                 shuffle=True,
                 partition = True):
        """"""
        self.lists = lists
        self.sampler = DistributedSampler(shuffle, partition)

    def set_epoch(self, epoch):
        self.sampler.set_epoch(epoch)

    def __iter__(self):
        """"""
        sample_info = self.sampler.update()
        indexs = self.sampler.sample(self.lists)
        for index in indexs:
            data = dict(src=self.lists[index])
            data.update(sample_info)
            yield data

def Dataset(
        data_type,
        data_list_file,
        tokenizer,
        conf,
        partition=True
):
    """"""
    assert data_type in ['raw', 'shard'], f'data_type is {data_type}'
    shuffle = conf.get('shuffle', True)
    data_list = utils_file.load_dict_from_scp(data_list_file)
    dataset = DataList(
        lists=data_list,
        shuffle=shuffle,
        partition=partition
    )
    if data_type == 'shard':
        dataset = Processor(dataset, processor.url_opener)
        dataset = Processor(dataset, processor.tar_file_and_group)
    else:
        dataset = Processor(dataset, processor.parse_raw)

    speaker_conf = conf.get('speaker_conf', None)
    if speaker_conf is not None:
        dataset = Processor(dataset, processor.parse_speaker, **speaker_conf)

    dataset = Processor(dataset, processor.tokenize, tokenizer)
    filter_conf = conf.get('filter_conf', {})
    dataset = Processor(dataset, processor.filter, **filter_conf)

    resample_conf = conf.get('resample_conf', {})
    dataset = Processor(dataset, processor.resample, **resample_conf)

    speed_perturb = conf.get('speed_perturb', False)
    if speed_perturb:
        dataset = Processor(dataset, processor.speed_perturb)
    noice_augment = conf.get('noice_augment', False)
    # 开启噪音增广
    if noice_augment:
        noice_augment_conf = conf.get('noice_augment_conf', {})
        dataset = Processor(dataset, processor.gxl_noise_augment, **noice_augment_conf)

    feats_type = conf.get('feats_type', 'fbank')
    assert feats_type in ['fbank', 'mfcc', 'log_mel_spectrogram']
    if feats_type == 'fbank':
        fbank_conf = conf.get('fbank_conf', {})
        dataset = Processor(dataset, processor.compute_fbank, **fbank_conf)
    elif feats_type == 'mfcc':
        mfcc_conf = conf.get('mfcc_conf', {})
        dataset = Processor(dataset, processor.compute_mfcc, **mfcc_conf)
    elif feats_type == 'log_mel_spectrogram':
        log_mel_spectrogram_conf = conf.get('log_mel_spectrogram_conf', {})
        dataset = Processor(dataset, processor.compute_log_mel_spectrogram,
                            **log_mel_spectrogram_conf)

    spec_aug = conf.get('spec_aug', True)
    spec_sub = conf.get('spec_sub', False)
    spec_trim = conf.get('spec_trim', False)
    if spec_aug:
        spec_aug_conf = conf.get('spec_aug_conf', {})
        dataset = Processor(dataset, processor.spec_aug, **spec_aug_conf)
    if spec_sub:
        spec_sub_conf = conf.get('spec_sub_conf', {})
        dataset = Processor(dataset, processor.spec_sub, **spec_sub_conf)
    if spec_trim:
        spec_trim_conf = conf.get('spec_trim_conf', {})
        dataset = Processor(dataset, processor.spec_trim, **spec_trim_conf)

    if shuffle:
        shuffle_conf = conf.get('shuffle_conf', {})
        dataset = Processor(dataset, processor.shuffle, **shuffle_conf)

    sort = conf.get('sort', True)
    if sort:
        sort_conf = conf.get('sort_conf', {})
        dataset = Processor(dataset, processor.sort, **sort_conf)

    batch_conf = conf.get('batch_conf', {})
    dataset = Processor(dataset, processor.batch, **batch_conf)
    dataset = Processor(dataset, processor.padding)
    return dataset




if __name__ == '__main__':
    fake_list = []
    for i in range(10000):
        fake_list.append(str(i).zfill(7))
    dataset = DataList(fake_list)
    dataloader = torch.utils.data.DataLoader(dataset, batch_size=None, num_workers=4)
    for epoch_i in range(5):
        for data in dataloader:
            print(data)






#!/usr/bin/env python3
# Copyright    2021  Xiaomi Corp.        (authors: Fangjun Kuang)
#
# See ../../../../LICENSE for clarification regarding multiple authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""
This file computes fbank features of the aishell dataset.
It looks for manifests in the directory data/manifests.

The generated fbank features are saved in data/fbank.
"""

import argparse
import logging
import os
from pathlib import Path
import sys

from gxl_ai_utils.utils import utils_file

sys.path.insert(0,'/home/work_nfs8/xlgeng/new_workspace/icefall')

import torch
from lhotse import (
    CutSet,
    Fbank,
    FbankConfig,
    LilcomChunkyWriter,
    WhisperFbank,
    WhisperFbankConfig,
)
from lhotse.recipes.utils import read_manifests_if_cached

from icefall.utils import get_executor, str2bool

# Torch's multithreaded behavior needs to be disabled or
# it wastes a lot of CPU and slow things down.
# Do this outside of main() in case it needs to take effect
# even when we are not invoking the main (e.g. when spawning subprocesses).


def compute_fbank_gxldata(
    num_mel_bins: int = 80,
    perturb_speed: bool = False,
    whisper_fbank: bool = False,
    fbank_dir: str = "data/fbank",
    manifests_dir: str = "data/manifests",
    prefix: str = "gxldata",
    partition: str = "train", # train dev test
):
    torch.set_num_threads(1)
    torch.set_num_interop_threads(1)
    utils_file.makedir_sil(fbank_dir)
    src_dir = Path(manifests_dir)
    output_dir = Path(fbank_dir)
    num_jobs = min(15, os.cpu_count())
    dataset_parts = (
        f"{partition}",
    )
    prefix = prefix
    suffix = "jsonl.gz"
    manifests = read_manifests_if_cached(
        dataset_parts=dataset_parts,
        output_dir=src_dir,
        prefix=prefix,
        suffix=suffix,
    )
    assert manifests is not None

    assert len(manifests) == len(dataset_parts), (
        len(manifests),
        len(dataset_parts),
        list(manifests.keys()),
        dataset_parts,
    )
    if whisper_fbank:
        extractor = WhisperFbank(
            WhisperFbankConfig(num_filters=num_mel_bins, device="cuda")
        )
    else:
        extractor = Fbank(FbankConfig(num_mel_bins=num_mel_bins))

    with get_executor() as ex:  # Initialize the executor only once.
        for partition, m in manifests.items():
            if (output_dir / f"{prefix}_cuts_{partition}.{suffix}").is_file():
                logging.info(f"{partition} already exists - skipping.")
                continue
            logging.info(f"Processing {partition}")
            cut_set = CutSet.from_manifests(
                recordings=m["recordings"],
                supervisions=m["supervisions"],
            )
            if "train" in partition and perturb_speed:
                logging.info("Doing speed perturb")
                cut_set = (
                    cut_set + cut_set.perturb_speed(0.9) + cut_set.perturb_speed(1.1)
                )
            cut_set = cut_set.compute_and_store_features(
                extractor=extractor,
                storage_path=f"{output_dir}/{prefix}_feats_{partition}",
                # when an executor is specified, make more partitions
                num_jobs=num_jobs if ex is None else 80,
                executor=ex,
                storage_type=LilcomChunkyWriter,
            )
            cut_set.to_file(output_dir / f"{prefix}_cuts_{partition}.{suffix}")


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--num-mel-bins",
        type=int,
        default=80,
        help="""The number of mel bins for Fbank""",
    )
    parser.add_argument(
        "--perturb-speed",
        type=str2bool,
        default=True,
        help="Enable 0.9 and 1.1 speed perturbation for data augmentation. Default: False.",
    )
    parser.add_argument(
        "--whisper-fbank",
        type=str2bool,
        default=False,
        help="Use WhisperFbank instead of Fbank. Default: False.",
    )
    parser.add_argument(
        "--output-dir",
        type=str,
        default="gxl_data/fbank_common",
        help="Output directory. Default: data/fbank.",
    )
    parser.add_argument(
        "--src-dir",
        type=str,
        default="gxl_data/manifest",
        help="清单文件的位置",
    )
    return parser.parse_args()


if __name__ == "__main__":
    formatter = "%(asctime)s %(levelname)s [%(filename)s:%(lineno)d] %(message)s"

    logging.basicConfig(format=formatter, level=logging.INFO)

    args = get_args()
    root_dir = "/home/work_nfs8/xlgeng/new_workspace/icefall/egs/aishell/ASR"
    compute_fbank_gxldata(
        num_mel_bins=args.num_mel_bins,
        perturb_speed=args.perturb_speed,
        whisper_fbank=args.whisper_fbank,
        fbank_dir=os.path.join(root_dir, args.output_dir),
        manifests_dir=os.path.join(root_dir, args.src_dir),
    )



from funasr.models.frontend.wav_frontend import apply_lfr

from gxl_ai_utils.model_warehouse.paraformer import build_asr_model
import torch
from gxl_ai_utils.config.gxl_config import GxlNode


def wav_frontend_for_paraformer(input_data, input_len, lfr_m=7, lfr_n=6):
    """
    paraformer的模型要求的输入的waveform, 并降低帧率为1/7, 得到特征维度为560（80*7）
    改造成输入是fbank,只进行降低帧率
    :param input_data: (b, t, fbank)
    :param input_len: (b,)
    :return:
    """
    feats = []
    feats_lens = []
    for i in range(len(input_data)):
        mat = input_len[i]
        if lfr_m != 1 or lfr_n != 1:
            mat = apply_lfr(mat, lfr_m, lfr_n)
        feat_length = mat.size(0)
        feats.append(mat)
        feats_lens.append(feat_length)
    feats_lens = torch.as_tensor(feats_lens)



class GxlParaformerEncoder:
    """"""

    def __init__(self):
        """"""
        model_checkpoint = './output/paraformer/speech_paraformer-large_asr_nat-zh-cn-16k-common-vocab8404-pytorch/model.pt'
        param = torch.load(model_checkpoint, map_location='cpu')
        for key, value in param.items():
            print(key, value.shape)
        config_yaml = 'E:\gengxuelong_study\server_local_adapter\gxl_work\gxl_ai_utils\eggs\knowledge_distillation\output\paraformer\speech_paraformer-large_asr_nat-zh-cn-16k-common-vocab8404-pytorch\config.yaml'
        yaml_node = GxlNode.get_config_from_yaml(config_yaml)
        print(yaml_node)
        _model = build_asr_model.build_asr_model(yaml_node)
        self.encoder = _model.encoder



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2014-2016  Brno University of Technology (author: Karel Vesely)
# Licensed under the Apache License, Version 2.0 (the "License")

import numpy as np
import sys, os, re, gzip, struct

#################################################
# Adding kaldi tools to shell path,

# Select kaldi,
if not 'KALDI_ROOT' in os.environ:
    # Default! To change run python with 'export KALDI_ROOT=/some_dir python'
    os.environ['KALDI_ROOT'] = '/mnt/matylda5/iveselyk/Tools/kaldi-trunk'

# Add kaldi tools to path,
os.environ['PATH'] = os.popen(
    'echo $KALDI_ROOT/src/bin:$KALDI_ROOT/tools/openfst/bin:$KALDI_ROOT/src/fstbin/:$KALDI_ROOT/src/gmmbin/:$KALDI_ROOT/src/featbin/:$KALDI_ROOT/src/lm/:$KALDI_ROOT/src/sgmmbin/:$KALDI_ROOT/src/sgmm2bin/:$KALDI_ROOT/src/fgmmbin/:$KALDI_ROOT/src/latbin/:$KALDI_ROOT/src/nnetbin:$KALDI_ROOT/src/nnet2bin:$KALDI_ROOT/src/nnet3bin:$KALDI_ROOT/src/online2bin/:$KALDI_ROOT/src/ivectorbin/:$KALDI_ROOT/src/lmbin/'
).readline().strip() + ':' + os.environ['PATH']


#################################################
# Define all custom exceptions,
class UnsupportedDataType(Exception):
    pass


class UnknownVectorHeader(Exception):
    pass


class UnknownMatrixHeader(Exception):
    pass


class BadSampleSize(Exception):
    pass


class BadInputFormat(Exception):
    pass


class SubprocessFailed(Exception):
    pass


#################################################
# Data-type independent helper functions,


def open_or_fd(file, mode='rb'):
    """ fd = open_or_fd(file)
   Open file, gzipped file, pipe, or forward the file-descriptor.
   Eventually seeks in the 'file' argument contains ':offset' suffix.
  """
    offset = None
    try:
        # strip 'ark:' prefix from r{x,w}filename (optional),
        if re.search('^(ark|scp)(,scp|,b|,t|,n?f|,n?p|,b?o|,n?s|,n?cs)*:',
                     file):
            (prefix, file) = file.split(':', 1)
        # separate offset from filename (optional),
        if re.search(':[0-9]+$', file):
            (file, offset) = file.rsplit(':', 1)
        # input pipe?
        if file[-1] == '|':
            fd = popen(file[:-1], 'rb')  # custom,
        # output pipe?
        elif file[0] == '|':
            fd = popen(file[1:], 'wb')  # custom,
        # is it gzipped?
        elif file.split('.')[-1] == 'gz':
            fd = gzip.open(file, mode)
        # a normal file...
        else:
            fd = open(file, mode)
    except TypeError:
        # 'file' is opened file descriptor,
        fd = file
    # Eventually seek to offset,
    if offset != None: fd.seek(int(offset))
    return fd


# based on '/usr/local/lib/python3.4/os.py'
def popen(cmd, mode="rb"):
    if not isinstance(cmd, str):
        raise TypeError("invalid cmd type (%s, expected string)" % type(cmd))

    import subprocess, io, threading

    # cleanup function for subprocesses,
    def cleanup(proc, cmd):
        ret = proc.wait()
        if ret > 0:
            raise SubprocessFailed('cmd %s returned %d !' % (cmd, ret))
        return

    # text-mode,
    if mode == "r":
        proc = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
        threading.Thread(target=cleanup,
                         args=(proc, cmd)).start()  # clean-up thread,
        return io.TextIOWrapper(proc.stdout)
    elif mode == "w":
        proc = subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE)
        threading.Thread(target=cleanup,
                         args=(proc, cmd)).start()  # clean-up thread,
        return io.TextIOWrapper(proc.stdin)
    # binary,
    elif mode == "rb":
        proc = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
        threading.Thread(target=cleanup,
                         args=(proc, cmd)).start()  # clean-up thread,
        return proc.stdout
    elif mode == "wb":
        proc = subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE)
        threading.Thread(target=cleanup,
                         args=(proc, cmd)).start()  # clean-up thread,
        return proc.stdin
    # sanity,
    else:
        raise ValueError("invalid mode %s" % mode)


def read_key(fd):
    """ [key] = read_key(fd)
   Read the utterance-key from the opened ark/stream descriptor 'fd'.
  """
    key = ''
    while 1:
        char = fd.read(1).decode("latin1")
        if char == '': break
        if char == ' ': break
        key += char
    key = key.strip()
    if key == '': return None  # end of file,
    assert (re.match('^\S+$', key) != None)  # check format (no whitespace!)
    return key


#################################################
# Integer vectors (alignments, ...),


def read_ali_ark(file_or_fd):
    """ Alias to 'read_vec_int_ark()' """
    return read_vec_int_ark(file_or_fd)


def read_vec_int_ark(file_or_fd):
    """ generator(key,vec) = read_vec_int_ark(file_or_fd)
   Create generator of (key,vector<int>) tuples, which reads from the ark file/stream.
   file_or_fd : ark, gzipped ark, pipe or opened file descriptor.

   Read ark to a 'dictionary':
   d = { u:d for u,d in kaldi_io.read_vec_int_ark(file) }
  """
    fd = open_or_fd(file_or_fd)
    try:
        key = read_key(fd)
        while key:
            ali = read_vec_int(fd)
            yield key, ali
            key = read_key(fd)
    finally:
        if fd is not file_or_fd: fd.close()


def read_vec_int_scp(file_or_fd):
    """ generator(key,vec) = read_vec_int_scp(file_or_fd)
   Returns generator of (key,vector<int>) tuples, read according to kaldi scp.
   file_or_fd : scp, gzipped scp, pipe or opened file descriptor.

   Iterate the scp:
   for key,vec in kaldi_io.read_vec_int_scp(file):
     ...

   Read scp to a 'dictionary':
   d = { key:vec for key,mat in kaldi_io.read_vec_int_scp(file) }
  """
    fd = open_or_fd(file_or_fd)
    try:
        for line in fd:
            (key, rxfile) = line.decode().split(' ')
            vec = read_vec_int(rxfile)
            yield key, vec
    finally:
        if fd is not file_or_fd: fd.close()


def read_vec_int(file_or_fd):
    """ [int-vec] = read_vec_int(file_or_fd)
   Read kaldi integer vector, ascii or binary input,
  """
    fd = open_or_fd(file_or_fd)
    binary = fd.read(2).decode()
    if binary == '\0B':  # binary flag
        assert (fd.read(1).decode() == '\4')
        # int-size
        vec_size = np.frombuffer(fd.read(4), dtype='int32',
                                 count=1)[0]  # vector dim
        # Elements from int32 vector are sored in tuples: (sizeof(int32), value),
        vec = np.frombuffer(fd.read(vec_size * 5),
                            dtype=[('size', 'int8'), ('value', 'int32')],
                            count=vec_size)
        assert (vec[0]['size'] == 4)  # int32 size,
        ans = vec[:]['value']  # values are in 2nd column,
    else:  # ascii,
        arr = (binary + fd.readline().decode()).strip().split()
        try:
            arr.remove('[')
            arr.remove(']')  # optionally
        except ValueError:
            pass
        ans = np.array(arr, dtype=int)
    if fd is not file_or_fd: fd.close()  # cleanup
    return ans


# Writing,
def write_vec_int(file_or_fd, v, key=''):
    """ write_vec_int(f, v, key='')
   Write a binary kaldi integer vector to filename or stream.
   Arguments:
   file_or_fd : filename or opened file descriptor for writing,
   v : the vector to be stored,
   key (optional) : used for writing ark-file, the utterance-id gets written before the vector.

   Example of writing single vector:
   kaldi_io.write_vec_int(filename, vec)

   Example of writing arkfile:
   with open(ark_file,'w') as f:
     for key,vec in dict.iteritems():
       kaldi_io.write_vec_flt(f, vec, key=key)
  """
    fd = open_or_fd(file_or_fd, mode='wb')
    if sys.version_info[0] == 3: assert (fd.mode == 'wb')
    try:
        if key != '':
            fd.write(
                (key +
                 ' ').encode("latin1"))  # ark-files have keys (utterance-id),
        fd.write('\0B'.encode())  # we write binary!
        # dim,
        fd.write('\4'.encode())  # int32 type,
        fd.write(struct.pack(np.dtype('int32').char, v.shape[0]))
        # data,
        for i in range(len(v)):
            fd.write('\4'.encode())  # int32 type,
            fd.write(struct.pack(np.dtype('int32').char, v[i]))  # binary,
    finally:
        if fd is not file_or_fd: fd.close()


#################################################
# Float vectors (confidences, ivectors, ...),


# Reading,
def read_vec_flt_scp(file_or_fd):
    """ generator(key,mat) = read_vec_flt_scp(file_or_fd)
   Returns generator of (key,vector) tuples, read according to kaldi scp.
   file_or_fd : scp, gzipped scp, pipe or opened file descriptor.

   Iterate the scp:
   for key,vec in kaldi_io.read_vec_flt_scp(file):
     ...

   Read scp to a 'dictionary':
   d = { key:mat for key,mat in kaldi_io.read_mat_scp(file) }
  """
    fd = open_or_fd(file_or_fd)
    try:
        for line in fd:
            (key, rxfile) = line.decode().split(' ')
            vec = read_vec_flt(rxfile)
            yield key, vec
    finally:
        if fd is not file_or_fd: fd.close()


def read_vec_flt_ark(file_or_fd):
    """ generator(key,vec) = read_vec_flt_ark(file_or_fd)
   Create generator of (key,vector<float>) tuples, reading from an ark file/stream.
   file_or_fd : ark, gzipped ark, pipe or opened file descriptor.

   Read ark to a 'dictionary':
   d = { u:d for u,d in kaldi_io.read_vec_flt_ark(file) }
  """
    fd = open_or_fd(file_or_fd)
    try:
        key = read_key(fd)
        while key:
            ali = read_vec_flt(fd)
            yield key, ali
            key = read_key(fd)
    finally:
        if fd is not file_or_fd: fd.close()


def read_vec_flt(file_or_fd):
    """ [flt-vec] = read_vec_flt(file_or_fd)
   Read kaldi float vector, ascii or binary input,
  """
    fd = open_or_fd(file_or_fd)
    binary = fd.read(2).decode()
    if binary == '\0B':  # binary flag
        # Data type,
        header = fd.read(3).decode()
        if header == 'FV ': sample_size = 4  # floats
        elif header == 'DV ': sample_size = 8  # doubles
        else: raise UnknownVectorHeader("The header contained '%s'" % header)
        assert (sample_size > 0)
        # Dimension,
        assert (fd.read(1).decode() == '\4')
        # int-size
        vec_size = np.frombuffer(fd.read(4), dtype='int32',
                                 count=1)[0]  # vector dim
        # Read whole vector,
        buf = fd.read(vec_size * sample_size)
        if sample_size == 4: ans = np.frombuffer(buf, dtype='float32')
        elif sample_size == 8: ans = np.frombuffer(buf, dtype='float64')
        else: raise BadSampleSize
        return ans
    else:  # ascii,
        arr = (binary + fd.readline().decode()).strip().split()
        try:
            arr.remove('[')
            arr.remove(']')  # optionally
        except ValueError:
            pass
        ans = np.array(arr, dtype=float)
    if fd is not file_or_fd: fd.close()  # cleanup
    return ans


# Writing,
def write_vec_flt(file_or_fd, v, key=''):
    """ write_vec_flt(f, v, key='')
   Write a binary kaldi vector to filename or stream. Supports 32bit and 64bit floats.
   Arguments:
   file_or_fd : filename or opened file descriptor for writing,
   v : the vector to be stored,
   key (optional) : used for writing ark-file, the utterance-id gets written before the vector.

   Example of writing single vector:
   kaldi_io.write_vec_flt(filename, vec)

   Example of writing arkfile:
   with open(ark_file,'w') as f:
     for key,vec in dict.iteritems():
       kaldi_io.write_vec_flt(f, vec, key=key)
  """
    fd = open_or_fd(file_or_fd, mode='wb')
    if sys.version_info[0] == 3: assert (fd.mode == 'wb')
    try:
        if key != '':
            fd.write(
                (key +
                 ' ').encode("latin1"))  # ark-files have keys (utterance-id),
        fd.write('\0B'.encode())  # we write binary!
        # Data-type,
        if v.dtype == 'float32': fd.write('FV '.encode())
        elif v.dtype == 'float64': fd.write('DV '.encode())
        else:
            raise UnsupportedDataType(
                "'%s', please use 'float32' or 'float64'" % v.dtype)
        # Dim,
        fd.write('\04'.encode())
        fd.write(struct.pack(np.dtype('uint32').char, v.shape[0]))  # dim
        # Data,
        fd.write(v.tobytes())
    finally:
        if fd is not file_or_fd: fd.close()


#################################################
# Float matrices (features, transformations, ...),


# Reading,
def read_mat_scp(file_or_fd):
    """ generator(key,mat) = read_mat_scp(file_or_fd)
   Returns generator of (key,matrix) tuples, read according to kaldi scp.
   file_or_fd : scp, gzipped scp, pipe or opened file descriptor.

   Iterate the scp:
   for key,mat in kaldi_io.read_mat_scp(file):
     ...

   Read scp to a 'dictionary':
   d = { key:mat for key,mat in kaldi_io.read_mat_scp(file) }
  """
    fd = open_or_fd(file_or_fd)
    try:
        for line in fd:
            (key, rxfile) = line.decode().split(' ')
            mat = read_mat(rxfile)
            yield key, mat
    finally:
        if fd is not file_or_fd: fd.close()


def read_mat_ark(file_or_fd):
    """ generator(key,mat) = read_mat_ark(file_or_fd)
   Returns generator of (key,matrix) tuples, read from ark file/stream.
   file_or_fd : scp, gzipped scp, pipe or opened file descriptor.

   Iterate the ark:
   for key,mat in kaldi_io.read_mat_ark(file):
     ...

   Read ark to a 'dictionary':
   d = { key:mat for key,mat in kaldi_io.read_mat_ark(file) }
  """
    fd = open_or_fd(file_or_fd)
    try:
        key = read_key(fd)
        while key:
            mat = read_mat(fd)
            yield key, mat
            key = read_key(fd)
    finally:
        if fd is not file_or_fd: fd.close()


def read_mat(file_or_fd):
    """ [mat] = read_mat(file_or_fd)
   Reads single kaldi matrix, supports ascii and binary.
   file_or_fd : file, gzipped file, pipe or opened file descriptor.
  """
    fd = open_or_fd(file_or_fd)
    try:
        binary = fd.read(2).decode()
        if binary == '\0B':
            mat = _read_mat_binary(fd)
        else:
            assert (binary == ' [')
            mat = _read_mat_ascii(fd)
    finally:
        if fd is not file_or_fd: fd.close()
    return mat


def _read_mat_binary(fd):
    # Data type
    header = fd.read(3).decode()
    # 'CM', 'CM2', 'CM3' are possible values,
    if header.startswith('CM'): return _read_compressed_mat(fd, header)
    elif header == 'FM ': sample_size = 4  # floats
    elif header == 'DM ': sample_size = 8  # doubles
    else: raise UnknownMatrixHeader("The header contained '%s'" % header)
    assert (sample_size > 0)
    # Dimensions
    s1, rows, s2, cols = np.frombuffer(fd.read(10),
                                       dtype='int8,int32,int8,int32',
                                       count=1)[0]
    # Read whole matrix
    buf = fd.read(rows * cols * sample_size)
    if sample_size == 4: vec = np.frombuffer(buf, dtype='float32')
    elif sample_size == 8: vec = np.frombuffer(buf, dtype='float64')
    else: raise BadSampleSize
    mat = np.reshape(vec, (rows, cols))
    return mat


def _read_mat_ascii(fd):
    rows = []
    while 1:
        line = fd.readline().decode()
        if (len(line) == 0): raise BadInputFormat  # eof, should not happen!
        if len(line.strip()) == 0: continue  # skip empty line
        arr = line.strip().split()
        if arr[-1] != ']':
            rows.append(np.array(arr, dtype='float32'))  # not last line
        else:
            rows.append(np.array(arr[:-1], dtype='float32'))  # last line
            mat = np.vstack(rows)
            return mat


def _read_compressed_mat(fd, format):
    """ Read a compressed matrix,
      see: https://github.com/kaldi-asr/kaldi/blob/master/src/matrix/compressed-matrix.h
      methods: CompressedMatrix::Read(...), CompressedMatrix::CopyToMat(...),
  """
    assert (format == 'CM ')  # The formats CM2, CM3 are not supported...

    # Format of header 'struct',
    global_header = np.dtype([('minvalue', 'float32'), ('range', 'float32'),
                              ('num_rows', 'int32'), ('num_cols', 'int32')
                              ])  # member '.format' is not written,
    per_col_header = np.dtype([('percentile_0', 'uint16'),
                               ('percentile_25', 'uint16'),
                               ('percentile_75', 'uint16'),
                               ('percentile_100', 'uint16')])

    # Mapping for percentiles in col-headers,
    def uint16_to_float(value, min, range):
        return np.float32(min + range * 1.52590218966964e-05 * value)

    # Mapping for matrix elements,
    def uint8_to_float_v2(vec, p0, p25, p75, p100):
        # Split the vector by masks,
        mask_0_64 = (vec <= 64)
        mask_193_255 = (vec > 192)
        mask_65_192 = (~(mask_0_64 | mask_193_255))
        # Sanity check (useful but slow...),
        # assert(len(vec) == np.sum(np.hstack([mask_0_64,mask_65_192,mask_193_255])))
        # assert(len(vec) == np.sum(np.any([mask_0_64,mask_65_192,mask_193_255], axis=0)))
        # Build the float vector,
        ans = np.empty(len(vec), dtype='float32')
        ans[mask_0_64] = p0 + (p25 - p0) / 64. * vec[mask_0_64]
        ans[mask_65_192] = p25 + (p75 - p25) / 128. * (vec[mask_65_192] - 64)
        ans[mask_193_255] = p75 + (p100 - p75) / 63. * (vec[mask_193_255] -
                                                        192)
        return ans

    # Read global header,
    globmin, globrange, rows, cols = np.frombuffer(fd.read(16),
                                                   dtype=global_header,
                                                   count=1)[0]

    # The data is structed as [Colheader, ... , Colheader, Data, Data , .... ]
    #                         {           cols           }{     size         }
    col_headers = np.frombuffer(fd.read(cols * 8),
                                dtype=per_col_header,
                                count=cols)
    data = np.reshape(np.frombuffer(fd.read(cols * rows),
                                    dtype='uint8',
                                    count=cols * rows),
                      newshape=(cols, rows))  # stored as col-major,

    mat = np.empty((cols, rows), dtype='float32')
    for i, col_header in enumerate(col_headers):
        col_header_flt = [
            uint16_to_float(percentile, globmin, globrange)
            for percentile in col_header
        ]
        mat[i] = uint8_to_float_v2(data[i], *col_header_flt)

    return mat.T  # transpose! col-major -> row-major,


def write_ark_scp(key, mat, ark_fout, scp_out):
    mat_offset = write_mat(ark_fout, mat, key)
    scp_line = '{}\t{}:{}'.format(key, ark_fout.name, mat_offset)
    scp_out.write(scp_line)
    scp_out.write('\n')


# Writing,
def write_mat(file_or_fd, m, key=''):
    """ write_mat(f, m, key='')
  Write a binary kaldi matrix to filename or stream. Supports 32bit and 64bit floats.
  Arguments:
   file_or_fd : filename of opened file descriptor for writing,
   m : the matrix to be stored,
   key (optional) : used for writing ark-file, the utterance-id gets written before the matrix.

   Example of writing single matrix:
   kaldi_io.write_mat(filename, mat)

   Example of writing arkfile:
   with open(ark_file,'w') as f:
     for key,mat in dict.iteritems():
       kaldi_io.write_mat(f, mat, key=key)
  """
    mat_offset = 0
    fd = open_or_fd(file_or_fd, mode='wb')
    if sys.version_info[0] == 3: assert (fd.mode == 'wb')
    try:
        if key != '':
            fd.write(
                (key +
                 ' ').encode("latin1"))  # ark-files have keys (utterance-id),
        mat_offset = fd.tell()
        fd.write('\0B'.encode())  # we write binary!
        # Data-type,
        if m.dtype == 'float32': fd.write('FM '.encode())
        elif m.dtype == 'float64': fd.write('DM '.encode())
        else:
            raise UnsupportedDataType(
                "'%s', please use 'float32' or 'float64'" % m.dtype)
        # Dims,
        fd.write('\04'.encode())
        fd.write(struct.pack(np.dtype('uint32').char, m.shape[0]))  # rows
        fd.write('\04'.encode())
        fd.write(struct.pack(np.dtype('uint32').char, m.shape[1]))  # cols
        # Data,
        fd.write(m.tobytes())
    finally:
        if fd is not file_or_fd: fd.close()
    return mat_offset


#################################################
# 'Posterior' kaldi type (posteriors, confusion network, nnet1 training targets, ...)
# Corresponds to: vector<vector<tuple<int,float> > >
# - outer vector: time axis
# - inner vector: records at the time
# - tuple: int = index, float = value
#


def read_cnet_ark(file_or_fd):
    """ Alias of function 'read_post_ark()', 'cnet' = confusion network """
    return read_post_ark(file_or_fd)


def read_post_ark(file_or_fd):
    """ generator(key,vec<vec<int,float>>) = read_post_ark(file)
   Returns generator of (key,posterior) tuples, read from ark file.
   file_or_fd : ark, gzipped ark, pipe or opened file descriptor.

   Iterate the ark:
   for key,post in kaldi_io.read_post_ark(file):
     ...

   Read ark to a 'dictionary':
   d = { key:post for key,post in kaldi_io.read_post_ark(file) }
  """
    fd = open_or_fd(file_or_fd)
    try:
        key = read_key(fd)
        while key:
            post = read_post(fd)
            yield key, post
            key = read_key(fd)
    finally:
        if fd is not file_or_fd: fd.close()


def read_post(file_or_fd):
    """ [post] = read_post(file_or_fd)
   Reads single kaldi 'Posterior' in binary format.

   The 'Posterior' is C++ type 'vector<vector<tuple<int,float> > >',
   the outer-vector is usually time axis, inner-vector are the records
   at given time,  and the tuple is composed of an 'index' (integer)
   and a 'float-value'. The 'float-value' can represent a probability
   or any other numeric value.

   Returns vector of vectors of tuples.
  """
    fd = open_or_fd(file_or_fd)
    ans = []
    binary = fd.read(2).decode()
    assert (binary == '\0B')
    # binary flag
    assert (fd.read(1).decode() == '\4')
    # int-size
    outer_vec_size = np.frombuffer(fd.read(4), dtype='int32',
                                   count=1)[0]  # number of frames (or bins)

    # Loop over 'outer-vector',
    for i in range(outer_vec_size):
        assert (fd.read(1).decode() == '\4')
        # int-size
        inner_vec_size = np.frombuffer(
            fd.read(4), dtype='int32',
            count=1)[0]  # number of records for frame (or bin)
        data = np.frombuffer(fd.read(inner_vec_size * 10),
                             dtype=[('size_idx', 'int8'), ('idx', 'int32'),
                                    ('size_post', 'int8'),
                                    ('post', 'float32')],
                             count=inner_vec_size)
        assert (data[0]['size_idx'] == 4)
        assert (data[0]['size_post'] == 4)
        ans.append(data[['idx', 'post']].tolist())

    if fd is not file_or_fd: fd.close()
    return ans


#################################################
# Kaldi Confusion Network bin begin/end times,
# (kaldi stores CNs time info separately from the Posterior).
#


def read_cntime_ark(file_or_fd):
    """ generator(key,vec<tuple<float,float>>) = read_cntime_ark(file_or_fd)
   Returns generator of (key,cntime) tuples, read from ark file.
   file_or_fd : file, gzipped file, pipe or opened file descriptor.

   Iterate the ark:
   for key,time in kaldi_io.read_cntime_ark(file):
     ...

   Read ark to a 'dictionary':
   d = { key:time for key,time in kaldi_io.read_post_ark(file) }
  """
    fd = open_or_fd(file_or_fd)
    try:
        key = read_key(fd)
        while key:
            cntime = read_cntime(fd)
            yield key, cntime
            key = read_key(fd)
    finally:
        if fd is not file_or_fd: fd.close()


def read_cntime(file_or_fd):
    """ [cntime] = read_cntime(file_or_fd)
   Reads single kaldi 'Confusion Network time info', in binary format:
   C++ type: vector<tuple<float,float> >.
   (begin/end times of bins at the confusion network).

   Binary layout is '<num-bins> <beg1> <end1> <beg2> <end2> ...'

   file_or_fd : file, gzipped file, pipe or opened file descriptor.

   Returns vector of tuples.
  """
    fd = open_or_fd(file_or_fd)
    binary = fd.read(2).decode()
    assert (binary == '\0B')
    # assuming it's binary

    assert (fd.read(1).decode() == '\4')
    # int-size
    vec_size = np.frombuffer(fd.read(4), dtype='int32',
                             count=1)[0]  # number of frames (or bins)

    data = np.frombuffer(fd.read(vec_size * 10),
                         dtype=[('size_beg', 'int8'), ('t_beg', 'float32'),
                                ('size_end', 'int8'), ('t_end', 'float32')],
                         count=vec_size)
    assert (data[0]['size_beg'] == 4)
    assert (data[0]['size_end'] == 4)
    ans = data[['t_beg',
                't_end']].tolist()  # Return vector of tuples (t_beg,t_end),

    if fd is not file_or_fd: fd.close()
    return ans


#################################################
# Segments related,
#


# Segments as 'Bool vectors' can be handy,
# - for 'superposing' the segmentations,
# - for frame-selection in Speaker-ID experiments,
def read_segments_as_bool_vec(segments_file):
    """ [ bool_vec ] = read_segments_as_bool_vec(segments_file)
   using kaldi 'segments' file for 1 wav, format : '<utt> <rec> <t-beg> <t-end>'
   - t-beg, t-end is in seconds,
   - assumed 100 frames/second,
  """
    segs = np.loadtxt(segments_file, dtype='object,object,f,f', ndmin=1)
    # Sanity checks,
    assert (len(segs) > 0)  # empty segmentation is an error,
    assert (len(np.unique([rec[1] for rec in segs])) == 1
            )  # segments with only 1 wav-file,
    # Convert time to frame-indexes,
    start = np.rint([100 * rec[2] for rec in segs]).astype(int)
    end = np.rint([100 * rec[3] for rec in segs]).astype(int)
    # Taken from 'read_lab_to_bool_vec', htk.py,
    frms = np.repeat(
        np.r_[np.tile([False, True], len(end)), False],
        np.r_[np.c_[start - np.r_[0, end[:-1]], end - start].flat, 0])
    assert np.sum(end - start) == np.sum(frms)
    return frms



import glob
import os
import random
import re
from urllib.parse import unquote

from gxl_ai_utils.utils import utils_spider, utils_file



class CommonSpider:
    def __init__(self):
        """"""
        self.url = 'https://voicewiki.cn'
        self.output_dir = "./output"
        self.person_infor_dict = None


    def re_draw_func(self,string):
        """"""
        bad_words = ['关于', '特殊', '分类', '免责', '隐私政策']
        # string = '<a href="aaaa" title="bbbb">'
        pattern = r'<a href="(.*?)" title="(.*?)">'
        matches = re.findall(pattern, string)
        res_dict = {}
        # 打印所有匹配的结果
        for match in matches:
            href = match[0]
            href = unquote(href, 'utf-8')
            title = match[1]
            if title in res_dict:
                continue
            href_key = href.split('/')[-1]
            jump_flag = False
            for bad_word in bad_words:
                if bad_word in href_key:
                    jump_flag = True
                    break
            if jump_flag:
                continue
            if href_key == title:
                res_dict[title] = self.url + href
        return res_dict

    def re_func_2(self, string):
        pattern = r'<div class="download-link"><a href="(.*?)" class="internal" title="(.*?)">(.*?)</a></div>'
        matches = re.findall(pattern, string)
        res_list = []
        # 打印所有匹配的结果
        for match in matches:
            href = match[0]
            href = unquote(href, 'utf-8')
            href = self.url + href
            res_list.append(href)
        return res_list
    def get_all_game_url(self,):
        output_path = os.path.join(self.output_dir, 'game_url.json')
        # 首页
        input_url = "https://voicewiki.cn/wiki/%E9%A6%96%E9%A1%B5"
        xpath = '//*[@id="mw-content-text"]/div/table[2]/tbody/tr[1]/td[1]/div/div/div[1]/a/@href'
        html_page = utils_spider.send_request(input_url).text
        tr_num_xpath = '//*[@id="mw-content-text"]/div/table[2]/tbody/tr'
        tr_num = len(utils_spider.handle_xpath(html_page, tr_num_xpath))
        td_num_xpath = '//*[@id="mw-content-text"]/div/table[2]/tbody/tr[1]/td'
        td_num = len(utils_spider.handle_xpath(html_page, td_num_xpath))
        res_dict = {}
        for i in range(1, tr_num + 1):
            for j in range(1, td_num + 1):
                game_url_xpath = f'//*[@id="mw-content-text"]/div/table[2]/tbody/tr[{i}]/td[{j}]/div/div/div[1]/a/@href'
                game_url = utils_spider.handle_xpath(html_page, game_url_xpath)
                if len(game_url) != 0:
                    url = self.url + game_url[0]
                    decoded_url = unquote(url, 'utf-8')
                    utils_file.logging_print(decoded_url)
                    key = decoded_url.split('/')[-1]
                    res_dict[key] = decoded_url
        utils_file.write_dict_to_json(res_dict, output_path)
        self.game_info_dict = res_dict

    def get_person_info_dict(self):
        """"""
        output_path = os.path.join(self.output_dir, 'person_infor_dict.json')
        person_infor_dict = {}
        for game_name, game_name_path in self.game_info_dict.items():
            """"""
            utils_file.logging_print(f"正在获取{game_name}的人物信息")
            str_html = utils_spider.send_request(game_name_path).text
            res_dict = self.re_draw_func(str_html)
            person_infor_dict[game_name] = res_dict
        utils_file.write_dict_to_json(person_infor_dict, output_path)
        self.person_infor_dict = person_infor_dict

    def get_wav_info_dict(self):
        """"""
        if self.person_infor_dict is None:
            self.person_infor_dict = utils_file.load_dict_from_json('./output/person_infor_dict_copy.json')
        output_path = os.path.join(self.output_dir, 'wav_info_dict.json')
        wav_info_output_dir = os.path.join(self.output_dir, 'wav_info')
        utils_file.makedir_sil(wav_info_output_dir)
        person_info_dir = os.path.join(self.output_dir, 'person_info')
        utils_file.makedir_sil(person_info_dir)
        wav_info_dict = {}
        for game_name, person_info_dict_i in self.person_infor_dict.items():
            try:
                game_info_i_dict = {}
                for person_name, person_page_url in person_info_dict_i.items():
                    try:
                        person_info_i_list = []
                        utils_file.logging_print(
                            f"正在获取{game_name}的{person_name}的音频信息, 地址：{person_page_url}")
                        html_str = utils_spider.send_request(person_page_url).text
                        if "守望先锋" in person_name:
                            """"""
                            res_list = []
                            # 守望先锋的游戏里 每个任务相当于个小游戏, 下面还有小人物
                            little_person_info_dict = self.re_draw_func(html_str)
                            for little_person_name, little_person_page_url in little_person_info_dict.items():
                                little_html_str = utils_spider.send_request(little_person_page_url).text
                                little_res_list = self.re_func_2(little_html_str)
                                res_list.extend(little_res_list)
                            person_info_i_list = res_list
                        else:
                            person_info_i_list = self.re_func_2(html_str)
                        game_info_i_dict[person_name] = person_info_i_list
                        output_path = os.path.join(wav_info_output_dir, f'{game_name}__{person_name}.list')
                        utils_file.write_list_to_file(person_info_i_list, output_path)
                    except Exception as e:
                        utils_file.logging_print(e)
                        continue
                wav_info_dict[game_name] = game_info_i_dict
                output_path = os.path.join(person_info_dir, f'{game_name}.json')
                utils_file.write_dict_to_json(game_info_i_dict, output_path)
            except Exception as e:
                utils_file.logging_print(e)
                continue


        all_person_info_dict_path_list = glob.glob(os.path.join(person_info_dir, '*.json'))
        for person_info_dict_path in all_person_info_dict_path_list:
            person_info_dict = utils_file.load_dict_from_json(person_info_dict_path)
            game_name = utils_file.get_file_pure_name_from_path(person_info_dict_path)
            wav_info_dict[game_name] = person_info_dict
        output_path = os.path.join(self.output_dir, 'wav_info_dict.json')
        utils_file.write_dict_to_json(wav_info_dict, output_path)








if __name__ == '__main__':
    spider = CommonSpider()
    # spider.get_all_game_url()
    # spider.get_person_info_dict()
    spider.get_wav_info_dict()


import os.path
import subprocess

import sys


sys.path.insert(0, '../../../../')
from gxl_ai_utils.utils import utils_file

def do_sync_files(file_list_path, password, username, remote_host, local_directory,remote_dir='/'):
    """
    使用 rsync 命令从远程主机下载文件，并支持断点续传(-P决定)。
    remote_dir使用的场景： 如果一个shards_list.txt文件中的pure文件名字不是唯一的， 则需要相对于某一个根目录，保留这个
    目录下面的子目录的结构，这样就不会出现覆盖文件的情况了。
    sync比scp更好用，功能相似的前体下sync支持断点续传
    :param file_list_path: 包含文件列表的文件路径
    :param password: 远程主机的 SSH 密码
    :param username: 远程主机的用户名
    :param remote_host: 远程主机的 IP 地址或主机名
    :param local_directory: 本地存储文件的目标目录
    :param remote_dir: 远程目录起始位置
    :return:
    """
    remote_dir = remote_dir if remote_dir.endswith('/') else remote_dir+'/'
    if len(remote_dir) !=1:
        utils_file.do_replace_str_to_file(remote_dir, '/', file_list_path, './tmp.list')
        file_list_path = './tmp.list'
        # 构造 rsync 命令
        rsync_command = [
            'sshpass', '-p', password,  # 使用 sshpass 提供密码
            'rsync', '-avrP',  # rsync 参数：-a（归档），-v（详细），-r（递归），-P（部分传输和进度）
            # '--no-relative',  # 去掉远程目录的层级结构
            f'--files-from={file_list_path}',  # 从文件列表中读取文件路径
            f'{username}@{remote_host}:{remote_dir}',  # 远程源路径
            local_directory  # 本地目标目录
        ]
    elif remote_dir=='/':
        # 构造 rsync 命令
        rsync_command = [
            'sshpass', '-p', password,  # 使用 sshpass 提供密码
            'rsync', '-avrP',  # rsync 参数：-a（归档），-v（详细），-r（递归），-P（部分传输和进度）
            '--no-relative',  # 去掉远程目录的层级结构
            f'--files-from={file_list_path}',  # 从文件列表中读取文件路径
            f'{username}@{remote_host}:{remote_dir}',  # 远程源路径
            local_directory  # 本地目标目录
        ]
    else:
        utils_file.logging_warning('remote dir格式不正确')
        return

    # 执行 rsync 命令
    try:
        subprocess.run(rsync_command, check=True)
        print("文件同步完成！")
        utils_file.do_remove_file('./tmp.list')
    except subprocess.CalledProcessError as e:
        print(f"rsync 命令执行失败: {e}")
    except FileNotFoundError as e:
        print(f"命令未找到: {e}")
    except Exception as e:
        print(f"发生错误: {e}")


# 示例调用
# file_list_path = "/home/work_nfs15/asr_data/data/LibriSpeech/LibriSpeech_shard/shards_list.txt"
password = 'ASLPaslp'
username = 'aslp'
remote_host = '192.168.0.4'
local_directory = '/home/node48_tmpdata/xlgeng/data/asr_data'

data_info_dict = utils_file.load_dict_from_yaml('data.yaml')
for key, data_info in data_info_dict.items():
    file_list_path = data_info['path']
    local_dir_i = os.path.join(local_directory, key)
    utils_file.makedir_sil(local_dir_i)
    if key == 'librispeech':
        remote_dir = "/home/work_nfs15/asr_data/data/LibriSpeech/LibriSpeech_shard/"
    else:
        remote_dir = '/'
    do_sync_files(file_list_path, password, username, remote_host, local_dir_i, remote_dir)
